@@ -106,6 +106,31 @@ emalloc2(nmemb, size)
     return ptr;
 }
 
+/*
+ * ecalloc() allocates nmemb * size bytes and exits with an error
+ * if overflow would occur or if the system malloc(3) fails.  
+ * On success, the allocated space is zero-filled.
+ */
+void *
+ecalloc(nmemb, size)
+    size_t nmemb;
+    size_t size;
+{
+    void *ptr;
+
+    if (nmemb == 0 || size == 0)
+	errorx(1, "internal error, tried to ecalloc(0)");
+    if (nmemb != 1) {
+	if (nmemb > SIZE_MAX / size)
+	    errorx(1, "internal error, ecalloc() overflow");
+	size *= nmemb;
+    }
+    if ((ptr = malloc(size)) == NULL)
+	errorx(1, "unable to allocate memory");
+    memset(ptr, 0, size);
+    return ptr;
+}
+
 /*
  * erealloc() calls the system realloc(3) and exits with an error if
  * realloc(3) fails.  You can call erealloc() with a NULL pointer even
@@ -171,6 +196,30 @@ estrdup(src)
     return dst;
 }
 
+/*
+ * estrdup() is like strndup(3) except that it exits with an error if
+ * malloc(3) fails.  NOTE: unlike strdup(3), estrdup(NULL) is legal.
+ */
+char *
+estrndup(src, maxlen)
+    const char *src;
+    size_t maxlen;
+{
+    char *dst = NULL;
+    size_t len = 0;
+
+    if (src != NULL) {
+	while (maxlen != 0 && src[len] != '\0') {
+	    len++;
+	    maxlen--;
+	}
+	dst = (char *) emalloc(len + 1);
+	(void) memcpy(dst, src, len);
+	dst[len] = '\0';
+    }
+    return dst;
+}
+
 /*
  * easprintf() calls vasprintf() and exits with an error if vasprintf()
  * returns -1 (out of memory).
