@@ -22,21 +22,25 @@
 int	 easprintf(char **, const char *, ...) __printflike(2, 3);
 int	 evasprintf(char **, const char *, va_list) __printflike(2, 0);
 void	 efree(void *);
+void	*ecalloc(size_t, size_t);
 void	*emalloc(size_t);
 void	*emalloc2(size_t, size_t);
 void	*erealloc(void *, size_t);
 void	*erealloc3(void *, size_t, size_t);
 char	*estrdup(const char *);
+char	*estrndup(const char *, size_t);
 #else
 # include <varargs.h>
 int	 easprintf();
 int	 evasprintf();
 void	 efree();
+void	*ecalloc();
 void	*emalloc();
 void	*emalloc2();
 void	*erealloc();
 void	*erealloc3();
 char	*estrdup();
+char	*estrndup();
 #endif /* __STDC__ */
 
 #endif /* _SUDO_ALLOC_H */
