@@ -75,13 +75,13 @@ bsdauth_init(pw, auth)
     else
        state.lc = login_getclass(pw->pw_uid ? LOGIN_DEFCLASS : LOGIN_DEFROOTCLASS);
     if (state.lc == NULL) {
-	log_error(USE_ERRNO|NO_EXIT|NO_MAIL,
+	log_error(USE_ERRNO|NO_MAIL,
 	    "unable to get login class for user %s", pw->pw_name);
 	return AUTH_FATAL;
     }
 
     if ((state.as = auth_open()) == NULL) {
-	log_error(USE_ERRNO|NO_EXIT|NO_MAIL,
+	log_error(USE_ERRNO|NO_MAIL,
 	    "unable to begin bsd authentication");
 	login_close(state.lc);
 	return AUTH_FATAL;
@@ -90,7 +90,7 @@ bsdauth_init(pw, auth)
     /* XXX - maybe sanity check the auth style earlier? */
     login_style = login_getstyle(state.lc, login_style, "auth-sudo");
     if (login_style == NULL) {
-	log_error(NO_EXIT|NO_MAIL, "invalid authentication type");
+	log_error(NO_MAIL, "invalid authentication type");
 	auth_close(state.as);
 	login_close(state.lc);
 	return AUTH_FATAL;
@@ -99,7 +99,7 @@ bsdauth_init(pw, auth)
      if (auth_setitem(state.as, AUTHV_STYLE, login_style) < 0 ||
 	auth_setitem(state.as, AUTHV_NAME, pw->pw_name) < 0 ||
 	auth_setitem(state.as, AUTHV_CLASS, login_class) < 0) {
-	log_error(NO_EXIT|NO_MAIL, "unable to setup authentication");
+	log_error(NO_MAIL, "unable to setup authentication");
 	auth_close(state.as);
 	login_close(state.lc);
 	return AUTH_FATAL;
@@ -173,7 +173,7 @@ bsdauth_verify(pw, prompt, auth)
 	return AUTH_INTR;
 
     if ((s = auth_getvalue(as, "errormsg")) != NULL)
-	log_error(NO_EXIT|NO_MAIL, "%s", s);
+	log_error(NO_MAIL, "%s", s);
     return AUTH_FAILURE;
 }
 
