@@ -99,13 +99,13 @@ sia_setup(pw, promptp, auth)
     if (sia_ses_init(&siah, Argc, Argv, NULL, pw->pw_name, ttyname(0), 1, NULL)
 	!= SIASUCCESS) {
 
-	log_error(USE_ERRNO|NO_EXIT|NO_MAIL,
+	log_error(USE_ERRNO|NO_MAIL,
 	    "unable to initialize SIA session");
-	return(AUTH_FATAL);
+	return AUTH_FATAL;
     }
 
     auth->data = (void *) siah;
-    return(AUTH_SUCCESS);
+    return AUTH_SUCCESS;
 }
 
 int
@@ -120,9 +120,9 @@ sia_verify(pw, prompt, auth)
 
     /* XXX - need a way to detect user hitting return or EOF at prompt */
     if (sia_ses_reauthent(sudo_collect, siah) == SIASUCCESS)
-	return(AUTH_SUCCESS);
+	return AUTH_SUCCESS;
     else
-	return(AUTH_FAILURE);
+	return AUTH_FAILURE;
 }
 
 int
@@ -133,5 +133,5 @@ sia_cleanup(pw, auth)
     SIAENTITY *siah = (SIAENTITY *) auth->data;
 
     (void) sia_ses_release(&siah);
-    return(AUTH_SUCCESS);
+    return AUTH_SUCCESS;
 }
