@@ -82,9 +82,10 @@ static struct tty_info {
     dev_t rdev;			/* tty device ID */
     ino_t ino;			/* tty inode number */
     struct timeval ctime;	/* tty inode change time */
+    pid_t sid;			/* ID of session with controlling tty */
 } tty_info;
 
-static void  build_timestamp	__P((char **, char **));
+static int   build_timestamp	__P((char **, char **));
 static int   timestamp_status	__P((char *, char *, char *, int));
 static char *expand_prompt	__P((char *, char *, char *));
 static void  lecture		__P((int));
@@ -93,10 +94,10 @@ static int   tty_is_devpts	__P((const char *));
 static struct passwd *get_authpw __P((void));
 
 /*
- * This function only returns if the user can successfully
- * verify who he/she is.
+ * Returns TRUE if the user successfully authenticates, FALSE if not
+ * or -1 on error.
  */
-void
+int
 check_user(validated, mode)
     int validated;
     int mode;
@@ -106,52 +107,64 @@ check_user(validated, mode)
     char *timestampfile = NULL;
     char *prompt;
     struct stat sb;
-    int status, need_pass = def_authenticate;
+    int status, rval = TRUE;
 
     /* Init authentication system regardless of whether we need a password. */
     auth_pw = get_authpw();
-    sudo_auth_init(auth_pw);
-
-    if (need_pass) {
-	/* Always need a password when -k was specified with the command. */
-	if (ISSET(mode, MODE_INVALIDATE)) {
-	    SET(validated, FLAG_CHECK_USER);
-	} else {
-	    /*
-	     * Don't prompt for the root passwd or if the user is exempt.
-	     * If the user is not changing uid/gid, no need for a password.
-	     */
-	    if (user_uid == 0 || (user_uid == runas_pw->pw_uid &&
-		(!runas_gr || user_in_group(sudo_user.pw, runas_gr->gr_name)))
-		|| user_is_exempt())
-		need_pass = FALSE;		
-	}
-    }
-    if (!need_pass)
+    if (sudo_auth_init(auth_pw) == -1) {
+	rval = -1;
 	goto done;
+    }
 
-    /* Stash the tty's ctime for tty ticket comparison. */
+    /*
+     * Don't prompt for the root passwd or if the user is exempt.
+     * If the user is not changing uid/gid, no need for a password.
+     */
+    if (!def_authenticate || user_uid == 0 || user_is_exempt())
+	goto done;
+    if (user_uid == runas_pw->pw_uid &&
+	(!runas_gr || user_in_group(sudo_user.pw, runas_gr->gr_name))) {
+#ifdef HAVE_SELINUX
+	if (user_role == NULL && user_type == NULL)
+#endif
+	    goto done;
+    }
+
+    /* Always need a password when -k was specified with the command. */
+    if (ISSET(mode, MODE_INVALIDATE))
+	SET(validated, FLAG_CHECK_USER);
+
+    /* Stash the tty's device, session ID and ctime for ticket comparison. */
     if (def_tty_tickets && user_ttypath && stat(user_ttypath, &sb) == 0) {
 	tty_info.dev = sb.st_dev;
 	tty_info.ino = sb.st_ino;
 	tty_info.rdev = sb.st_rdev;
 	if (tty_is_devpts(user_ttypath))
 	    ctim_get(&sb, &tty_info.ctime);
+	tty_info.sid = user_sid;
+    }
+
+    if (build_timestamp(&timestampdir, &timestampfile) == -1) {
+	rval = -1;
+	goto done;
     }
 
-    build_timestamp(&timestampdir, &timestampfile);
     status = timestamp_status(timestampdir, timestampfile, user_name,
 	TS_MAKE_DIRS);
 
     if (status != TS_CURRENT || ISSET(validated, FLAG_CHECK_USER)) {
 	/* Bail out if we are non-interactive and a password is required */
-	if (ISSET(mode, MODE_NONINTERACTIVE))
-	    errorx(1, "sorry, a password is required to run %s", getprogname());
+	if (ISSET(mode, MODE_NONINTERACTIVE)) {
+	    validated |= FLAG_NON_INTERACTIVE;
+	    log_auth_failure(validated, 0);
+	    rval = -1;
+	    goto done;
+	}
 
 	/* If user specified -A, make sure we have an askpass helper. */
 	if (ISSET(tgetpass_flags, TGP_ASKPASS)) {
 	    if (user_askpass == NULL)
-		log_error(NO_MAIL,
+		log_fatal(NO_MAIL,
 		    "no askpass program specified, try setting SUDO_ASKPASS");
 	} else if (!ISSET(tgetpass_flags, TGP_STDIN)) {
 	    /* If no tty but DISPLAY is set, use askpass if we have it. */
@@ -159,7 +172,7 @@ check_user(validated, mode)
 		if (user_askpass && user_display && *user_display != '\0') {
 		    SET(tgetpass_flags, TGP_ASKPASS);
 		} else if (!def_visiblepw) {
-		    log_error(NO_MAIL,
+		    log_fatal(NO_MAIL,
 			"no tty present and no askpass program specified");
 		}
 	    }
@@ -172,10 +185,11 @@ check_user(validated, mode)
 	prompt = expand_prompt(user_prompt ? user_prompt : def_passprompt,
 	    user_name, user_shost);
 
-	verify_user(auth_pw, prompt);
+	rval = verify_user(auth_pw, prompt, validated);
     }
     /* Only update timestamp if user was validated. */
-    if (ISSET(validated, VALIDATE_OK) && !ISSET(mode, MODE_INVALIDATE) && status != TS_ERROR)
+    if (rval == TRUE && ISSET(validated, VALIDATE_OK) &&
+	!ISSET(mode, MODE_INVALIDATE) && status != TS_ERROR)
 	update_timestamp(timestampdir, timestampfile);
     efree(timestampdir);
     efree(timestampfile);
@@ -183,6 +197,8 @@ check_user(validated, mode)
 done:
     sudo_auth_cleanup(auth_pw);
     pw_delref(auth_pw);
+
+    return rval;
 }
 
 /*
@@ -202,7 +218,7 @@ lecture(status)
 
     if (def_lecture_file && (fp = fopen(def_lecture_file, "r")) != NULL) {
 	while ((nread = fread(buf, sizeof(char), sizeof(buf), fp)) != 0)
-	    fwrite(buf, nread, 1, stderr);
+	    ignore_result(fwrite(buf, nread, 1, stderr));
 	fclose(fp);
     } else {
 	(void) fputs("\n\
@@ -236,17 +252,17 @@ update_timestamp(timestampdir, timestampfile)
 	 */
 	int fd = open(timestampfile, O_WRONLY|O_CREAT, 0600);
 	if (fd == -1)
-	    log_error(NO_EXIT|USE_ERRNO, "Can't open %s", timestampfile);
+	    log_error(USE_ERRNO, "Can't open %s", timestampfile);
 	else {
 	    lock_file(fd, SUDO_LOCK);
 	    if (write(fd, &tty_info, sizeof(tty_info)) != sizeof(tty_info))
-		log_error(NO_EXIT|USE_ERRNO, "Can't write %s", timestampfile);
+		log_error(USE_ERRNO, "Can't write %s", timestampfile);
 	    close(fd);
 	}
     } else {
 	if (touch(-1, timestampdir, NULL) == -1) {
 	    if (mkdir(timestampdir, 0700) == -1)
-		log_error(NO_EXIT|USE_ERRNO, "Can't mkdir %s", timestampdir);
+		log_error(USE_ERRNO, "Can't mkdir %s", timestampdir);
 	}
     }
     if (timestamp_uid != 0)
@@ -397,7 +413,7 @@ user_is_exempt()
 /*
  * Fills in timestampdir as well as timestampfile if using tty tickets.
  */
-static void
+static int
 build_timestamp(timestampdir, timestampfile)
     char **timestampdir;
     char **timestampfile;
@@ -408,7 +424,7 @@ build_timestamp(timestampdir, timestampfile)
     dirparent = def_timestampdir;
     len = easprintf(timestampdir, "%s/%s", dirparent, user_name);
     if (len >= PATH_MAX)
-	log_error(0, "timestamp path too long: %s", *timestampdir);
+	goto bad;
 
     /*
      * Timestamp file may be a file in the directory or NUL to use
@@ -427,14 +443,20 @@ build_timestamp(timestampdir, timestampfile)
 	else
 	    len = easprintf(timestampfile, "%s/%s/%s", dirparent, user_name, p);
 	if (len >= PATH_MAX)
-	    log_error(0, "timestamp path too long: %s", *timestampfile);
+	    goto bad;
     } else if (def_targetpw) {
 	len = easprintf(timestampfile, "%s/%s/%s", dirparent, user_name,
 	    runas_pw->pw_name);
 	if (len >= PATH_MAX)
-	    log_error(0, "timestamp path too long: %s", *timestampfile);
+	    goto bad;
     } else
 	*timestampfile = NULL;
+
+    return len;
+bad:
+    log_fatal(0, "timestamp path too long: %s",
+	*timestampfile ? *timestampfile : *timestampdir);
+    return -1;
 }
 
 /*
@@ -465,14 +487,14 @@ timestamp_status(timestampdir, timestampfile, user, flags)
      */
     if (lstat(dirparent, &sb) == 0) {
 	if (!S_ISDIR(sb.st_mode))
-	    log_error(NO_EXIT, "%s exists but is not a directory (0%o)",
+	    log_error(0, "%s exists but is not a directory (0%o)",
 		dirparent, (unsigned int) sb.st_mode);
 	else if (sb.st_uid != timestamp_uid)
-	    log_error(NO_EXIT, "%s owned by uid %u, should be uid %u",
+	    log_error(0, "%s owned by uid %u, should be uid %u",
 		dirparent, (unsigned int) sb.st_uid,
 		(unsigned int) timestamp_uid);
 	else if ((sb.st_mode & 0000022))
-	    log_error(NO_EXIT,
+	    log_error(0,
 		"%s writable by non-owner (0%o), should be mode 0700",
 		dirparent, (unsigned int) sb.st_mode);
 	else {
@@ -481,12 +503,12 @@ timestamp_status(timestampdir, timestampfile, user, flags)
 	    status = TS_MISSING;
 	}
     } else if (errno != ENOENT) {
-	log_error(NO_EXIT|USE_ERRNO, "can't stat %s", dirparent);
+	log_error(USE_ERRNO, "can't stat %s", dirparent);
     } else {
 	/* No dirparent, try to make one. */
 	if (ISSET(flags, TS_MAKE_DIRS)) {
 	    if (mkdir(dirparent, S_IRWXU))
-		log_error(NO_EXIT|USE_ERRNO, "can't mkdir %s",
+		log_error(USE_ERRNO, "can't mkdir %s",
 		    dirparent);
 	    else
 		status = TS_MISSING;
@@ -512,14 +534,14 @@ timestamp_status(timestampdir, timestampfile, user, flags)
 		if (unlink(timestampdir) == 0)
 		    status = TS_MISSING;
 	    } else
-		log_error(NO_EXIT, "%s exists but is not a directory (0%o)",
+		log_error(0, "%s exists but is not a directory (0%o)",
 		    timestampdir, (unsigned int) sb.st_mode);
 	} else if (sb.st_uid != timestamp_uid)
-	    log_error(NO_EXIT, "%s owned by uid %u, should be uid %u",
+	    log_error(0, "%s owned by uid %u, should be uid %u",
 		timestampdir, (unsigned int) sb.st_uid,
 		(unsigned int) timestamp_uid);
 	else if ((sb.st_mode & 0000022))
-	    log_error(NO_EXIT,
+	    log_error(0,
 		"%s writable by non-owner (0%o), should be mode 0700",
 		timestampdir, (unsigned int) sb.st_mode);
 	else {
@@ -528,7 +550,7 @@ timestamp_status(timestampdir, timestampfile, user, flags)
 	    status = TS_OLD;		/* do date check later */
 	}
     } else if (errno != ENOENT) {
-	log_error(NO_EXIT|USE_ERRNO, "can't stat %s", timestampdir);
+	log_error(USE_ERRNO, "can't stat %s", timestampdir);
     } else
 	status = TS_MISSING;
 
@@ -539,7 +561,7 @@ timestamp_status(timestampdir, timestampfile, user, flags)
     if (status == TS_MISSING && timestampfile && ISSET(flags, TS_MAKE_DIRS)) {
 	if (mkdir(timestampdir, S_IRWXU) == -1) {
 	    status = TS_ERROR;
-	    log_error(NO_EXIT|USE_ERRNO, "can't mkdir %s", timestampdir);
+	    log_error(USE_ERRNO, "can't mkdir %s", timestampdir);
 	}
     }
 
@@ -554,18 +576,18 @@ timestamp_status(timestampdir, timestampfile, user, flags)
 	if (lstat(timestampfile, &sb) == 0) {
 	    if (!S_ISREG(sb.st_mode)) {
 		status = TS_ERROR;
-		log_error(NO_EXIT, "%s exists but is not a regular file (0%o)",
+		log_error(0, "%s exists but is not a regular file (0%o)",
 		    timestampfile, (unsigned int) sb.st_mode);
 	    } else {
 		/* If bad uid or file mode, complain and kill the bogus file. */
 		if (sb.st_uid != timestamp_uid) {
-		    log_error(NO_EXIT,
+		    log_error(0,
 			"%s owned by uid %u, should be uid %u",
 			timestampfile, (unsigned int) sb.st_uid,
 			(unsigned int) timestamp_uid);
 		    (void) unlink(timestampfile);
 		} else if ((sb.st_mode & 0000022)) {
-		    log_error(NO_EXIT,
+		    log_error(0,
 			"%s writable by non-owner (0%o), should be mode 0600",
 			timestampfile, (unsigned int) sb.st_mode);
 		    (void) unlink(timestampfile);
@@ -596,7 +618,7 @@ timestamp_status(timestampdir, timestampfile, user, flags)
 		}
 	    }
 	} else if (errno != ENOENT) {
-	    log_error(NO_EXIT|USE_ERRNO, "can't stat %s", timestampfile);
+	    log_error(USE_ERRNO, "can't stat %s", timestampfile);
 	    status = TS_ERROR;
 	}
     }
@@ -619,7 +641,7 @@ timestamp_status(timestampdir, timestampfile, user, flags)
 		 */
 		if (mtime.tv_sec > now + 60 * def_timestamp_timeout * 2) {
 		    time_t tv_sec = (time_t)mtime.tv_sec;
-		    log_error(NO_EXIT,
+		    log_error(0,
 			"timestamp too far in the future: %20.20s",
 			4 + ctime(&tv_sec));
 		    if (timestampfile)
@@ -653,7 +675,9 @@ remove_timestamp(remove)
     char *timestampdir, *timestampfile, *path;
     int status;
 
-    build_timestamp(&timestampdir, &timestampfile);
+    if (build_timestamp(&timestampdir, &timestampfile) == -1)
+	return;
+
     status = timestamp_status(timestampdir, timestampfile, user_name,
 	TS_REMOVE);
     if (status == TS_OLD || status == TS_CURRENT) {
@@ -664,7 +688,7 @@ remove_timestamp(remove)
 	    else
 		status = rmdir(timestampdir);
 	    if (status == -1 && errno != ENOENT) {
-		log_error(NO_EXIT, "can't remove %s (%s), will reset to Epoch",
+		log_error(0, "can't remove %s (%s), will reset to Epoch",
 		    path, strerror(errno));
 		remove = FALSE;
 	    }
@@ -681,9 +705,9 @@ remove_timestamp(remove)
 }
 
 /*
- * Returns TRUE if tty lives on a devpts or /devices filesystem, else FALSE.
- * Unlike most filesystems, the ctime of devpts nodes is not updated when
- * the device node is written to, only when the inode's status changes,
+ * Returns TRUE if tty lives on a devpts, /dev or /devices filesystem, else
+ * FALSE.  Unlike most filesystems, the ctime of devpts nodes is not updated
+ * when the device node is written to, only when the inode's status changes,
  * typically via the chmod, chown, link, rename, or utimes system calls.
  * Since the ctime is "stable" in this case, we can stash it the tty ticket
  * file and use it to determine whether the tty ticket file is stale.
@@ -708,7 +732,7 @@ tty_is_devpts(tty)
     struct statvfs sfs;
 
     if (statvfs(tty, &sfs) == 0) {
-	if (strcmp(sfs.f_fstr, "devices") == 0)
+	if (strcmp(sfs.f_fstr, "dev") == 0 || strcmp(sfs.f_fstr, "devices") == 0)
 	    retval = TRUE;
     }
 #endif /* __linux__ */
@@ -727,13 +751,13 @@ get_authpw()
 
     if (def_rootpw) {
 	if ((pw = sudo_getpwuid(0)) == NULL)
-	    log_error(0, "unknown uid: 0");
+	    log_fatal(0, "unknown uid: 0");
     } else if (def_runaspw) {
 	if ((pw = sudo_getpwnam(def_runas_default)) == NULL)
-	    log_error(0, "unknown user: %s", def_runas_default);
+	    log_fatal(0, "unknown user: %s", def_runas_default);
     } else if (def_targetpw) {
 	if (runas_pw->pw_name == NULL)
-	    log_error(NO_MAIL|MSG_ONLY, "unknown uid: %u",
+	    log_fatal(NO_MAIL|MSG_ONLY, "unknown uid: %u",
 		(unsigned int) runas_pw->pw_uid);
 	pw_addref(runas_pw);
 	pw = runas_pw;
