@@ -439,7 +439,9 @@ init_defaults()
 #ifdef _PATH_SUDO_ASKPASS
     def_askpass = estrdup(_PATH_SUDO_ASKPASS);
 #endif
+#ifdef _PATH_SUDO_IO_LOGDIR
     def_iolog_dir = estrdup(_PATH_SUDO_IO_LOGDIR);
+#endif
     def_sudoers_locale = estrdup("C");
     def_env_reset = ENV_RESET;
     def_set_logname = TRUE;
@@ -550,6 +552,55 @@ update_defaults(what)
     return rc;
 }
 
+/*
+ * Check the defaults entries without actually setting them.
+ * Pass in an OR'd list of which default types to check.
+ */
+int
+check_defaults(what, quiet)
+    int what;
+    int quiet;
+{
+    struct sudo_defs_types *cur;
+    struct defaults *def;
+    int rc = TRUE;
+
+    tq_foreach_fwd(&defaults, def) {
+	switch (def->type) {
+	    case DEFAULTS:
+		if (!ISSET(what, SETDEF_GENERIC))
+		    continue;
+		break;
+	    case DEFAULTS_USER:
+		if (!ISSET(what, SETDEF_USER))
+		    continue;
+		break;
+	    case DEFAULTS_RUNAS:
+		if (!ISSET(what, SETDEF_RUNAS))
+		    continue;
+		break;
+	    case DEFAULTS_HOST:
+		if (!ISSET(what, SETDEF_HOST))
+		    continue;
+		break;
+	    case DEFAULTS_CMND:
+		if (!ISSET(what, SETDEF_CMND))
+		    continue;
+		break;
+	}
+	for (cur = sudo_defs_table; cur->name != NULL; cur++) {
+	    if (strcmp(def->var, cur->name) == 0)
+		break;
+	}
+	if (cur->name == NULL) {
+	    if (!quiet)
+		warningx("unknown defaults entry `%s'", def->var);
+	    rc = FALSE;
+	}
+    }
+    return rc;
+}
+
 static int
 store_int(val, def, op)
     char *val;
@@ -833,7 +884,7 @@ list_op(val, len, def, op)
 
     /* Add new node to the head of the list. */
     if (op == add) {
-	cur = emalloc(sizeof(struct list_member));
+	cur = ecalloc(1, sizeof(struct list_member));
 	cur->value = emalloc(len + 1);
 	(void) memcpy(cur->value, val, len);
 	cur->value[len] = '\0';
