@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2009-2010 Todd C. Miller <Todd.Miller@courtesan.com>
+ * Copyright (c) 2009-2012 Todd C. Miller <Todd.Miller@courtesan.com>
  *
  * Permission to use, copy, modify, and distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
@@ -84,10 +84,13 @@ static void schedule_signal __P((int signo));
 static int log_io;
 #endif /* _PATH_SUDO_IO_LOGDIR */
 
+static pid_t ppgrp = -1;
+volatile pid_t cmnd_pid = -1;
+
 static int handle_signals __P((int sv[2], pid_t child,
     struct command_status *cstat));
 #ifdef SA_SIGINFO
-static void handler_nofwd __P((int s, siginfo_t *info, void *context));
+static void handler_user_only __P((int s, siginfo_t *info, void *context));
 #endif
 
 /*
@@ -123,16 +126,39 @@ static int fork_cmnd(path, argv, envp, sv, rbac_enabled)
 {
     struct command_status cstat;
     sigaction_t sa;
-    pid_t child;
 
+    ppgrp = getpgrp();	/* parent's process group */
+
+    /*
+     * Handle suspend/restore of sudo and the command.
+     * In most cases, the command will be in the same process group as
+     * sudo and job control will "just work".  However, if the command
+     * changes its process group ID and does not change it back (or is
+     * kill by SIGSTOP which is not catchable), we need to resume the
+     * command manually.  Also, if SIGTSTP is sent directly to sudo,
+     * we need to suspend the command, and then suspend ourself, restoring
+     * the default SIGTSTP handler temporarily.
+     *
+     * XXX - currently we send SIGCONT upon resume in some cases where
+     * we don't need to (e.g. command pgrp == parent pgrp).
+     */
     zero_bytes(&sa, sizeof(sa));
     sigemptyset(&sa.sa_mask);
     sa.sa_flags = SA_INTERRUPT; /* do not restart syscalls */
+#ifdef SA_SIGINFO
+    sa.sa_flags |= SA_SIGINFO;
+    sa.sa_sigaction = handler;
+#else
     sa.sa_handler = handler;
+#endif
     sigaction(SIGCONT, &sa, NULL);
+#ifdef SA_SIGINFO
+    sa.sa_sigaction = handler_user_only;
+#endif
+    sigaction(SIGTSTP, &sa, NULL);
 
-    child = fork();
-    switch (child) {
+    cmnd_pid = fork();
+    switch (cmnd_pid) {
     case -1:
 	error(1, "fork");
 	break;
@@ -162,7 +188,7 @@ static int fork_cmnd(path, argv, envp, sv, rbac_enabled)
 	send(sv[1], &cstat, sizeof(cstat), 0);
 	_exit(1);
     }
-    return child;
+    return cmnd_pid;
 }
 
 static struct signal_state {
@@ -228,6 +254,7 @@ sudo_execve(path, argv, envp, uid, cstat, dowait, bgmode)
     int rbac_enabled = 0;
     fd_set *fdsr, *fdsw;
     sigaction_t sa;
+    sigset_t omask;
     pid_t child;
 
     /* If running in background mode, fork and exit. */
@@ -266,6 +293,7 @@ sudo_execve(path, argv, envp, uid, cstat, dowait, bgmode)
      * If we don't need to wait for the command to finish, just exec it.
      */
     if (!dowait) {
+	restore_signals();
 	exec_setup(FALSE, NULL, -1);
 	closefrom(def_closefrom);
 	my_execve(path, argv, envp);
@@ -296,7 +324,12 @@ sudo_execve(path, argv, envp, uid, cstat, dowait, bgmode)
      * Note: HP-UX select() will not be interrupted if SA_RESTART set.
      */
     sa.sa_flags = SA_INTERRUPT; /* do not restart syscalls */
+#ifdef SA_SIGINFO
+    sa.sa_flags |= SA_SIGINFO;
+    sa.sa_sigaction = handler;
+#else
     sa.sa_handler = handler;
+#endif
     sigaction(SIGALRM, &sa, NULL);
     sigaction(SIGCHLD, &sa, NULL);
     sigaction(SIGPIPE, &sa, NULL);
@@ -317,7 +350,7 @@ sudo_execve(path, argv, envp, uid, cstat, dowait, bgmode)
 # endif
     {
 	sa.sa_flags |= SA_SIGINFO;
-	sa.sa_sigaction = handler_nofwd;
+	sa.sa_sigaction = handler_user_only;
     }
 #endif /* SA_SIGINFO */
     sigaction(SIGHUP, &sa, NULL);
@@ -333,7 +366,7 @@ sudo_execve(path, argv, envp, uid, cstat, dowait, bgmode)
      */
 #ifdef _PATH_SUDO_IO_LOGDIR
     if (log_io)
-	child = fork_pty(path, argv, envp, sv, rbac_enabled, bgmode, &maxfd);
+	child = fork_pty(path, argv, envp, sv, rbac_enabled, bgmode, &maxfd, &omask);
     else
 #endif
 	child = fork_cmnd(path, argv, envp, sv, rbac_enabled);
@@ -351,11 +384,11 @@ sudo_execve(path, argv, envp, uid, cstat, dowait, bgmode)
      * In the event loop we pass input from user tty to master
      * and pass output from master to stdout and IO plugin.
      */
-    fdsr = (fd_set *)emalloc2(howmany(maxfd + 1, NFDBITS), sizeof(fd_mask));
-    fdsw = (fd_set *)emalloc2(howmany(maxfd + 1, NFDBITS), sizeof(fd_mask));
+    fdsr = emalloc2(howmany(maxfd + 1, NFDBITS), sizeof(fd_mask));
+    fdsw = emalloc2(howmany(maxfd + 1, NFDBITS), sizeof(fd_mask));
     for (;;) {
-	zero_bytes(fdsw, howmany(maxfd + 1, NFDBITS) * sizeof(fd_mask));
-	zero_bytes(fdsr, howmany(maxfd + 1, NFDBITS) * sizeof(fd_mask));
+	memset(fdsw, 0, howmany(maxfd + 1, NFDBITS) * sizeof(fd_mask));
+	memset(fdsr, 0, howmany(maxfd + 1, NFDBITS) * sizeof(fd_mask));
 
 	FD_SET(signal_pipe[0], fdsr);
 	FD_SET(sv[0], fdsr);
@@ -367,9 +400,20 @@ sudo_execve(path, argv, envp, uid, cstat, dowait, bgmode)
 #endif
 	nready = select(maxfd + 1, fdsr, fdsw, NULL, NULL);
 	if (nready == -1) {
-	    if (errno == EINTR)
+	    if (errno == EINTR || errno == ENOMEM)
 		continue;
-	    error(1, "select failed");
+#ifdef _PATH_SUDO_IO_LOGDIR
+	    if (errno == EBADF || errno == EIO) {
+		/* One of the ttys must have gone away. */
+		goto do_tty_io;
+	    }
+#endif
+	    warning("select failed");
+#ifdef _PATH_SUDO_IO_LOGDIR
+	    schedule_signal(SIGKILL);
+	    forward_signals(sv[0]);
+#endif
+	    break;
 	}
 #ifdef _PATH_SUDO_IO_LOGDIR
 	if (FD_ISSET(sv[0], fdsw)) {
@@ -406,15 +450,25 @@ sudo_execve(path, argv, envp, uid, cstat, dowait, bgmode)
 			cstat->val = errno;
 			break;
 		    }
+#endif
 		} else {
 		    /* Short read or EOF. */
 		    /* XXX - should set cstat */
 		    break;
 		}
-#endif
 	    }
 #ifdef _PATH_SUDO_IO_LOGDIR /* XXX */
-	    if (cstat->type == CMD_WSTATUS) {
+	    if (cstat->type == CMD_PID) {
+		/*
+		 * Once we know the command's pid we can unblock
+		 * signals which were blocked in fork_pty().  This
+		 * avoids a race between exec of the command and
+		 * receipt of a fatal signal from it.
+		 */
+		cmnd_pid = cstat->val;
+		if (log_io)
+		    sigprocmask(SIG_SETMASK, &omask, NULL);
+	    } else if (cstat->type == CMD_WSTATUS) {
 		if (WIFSTOPPED(cstat->val)) {
 		    /* Suspend parent and tell child how to resume on return. */
 		    n = suspend_parent(WSTOPSIG(cstat->val));
@@ -433,6 +487,7 @@ sudo_execve(path, argv, envp, uid, cstat, dowait, bgmode)
 	}
 
 #ifdef _PATH_SUDO_IO_LOGDIR
+do_tty_io:
 	if (perform_io(fdsr, fdsw, cstat) != 0) {
 	    /* I/O error, kill child if still alive and finish. */
 	    schedule_signal(SIGKILL);
@@ -532,18 +587,39 @@ handle_signals(sv, child, cstat)
 		    if (WIFSTOPPED(status)) {
 			/*
 			 * Save the controlling terminal's process group
-			 * so we can restore it after we resume.
+			 * so we can restore it after we resume, if needed.
+			 * Most well-behaved shells change the pgrp back to
+			 * its original value before suspending so we must
+			 * not try to restore in that case, lest we race with
+			 * the child upon resume, potentially stopping sudo
+			 * with SIGTTOU while the command continues to run.
 			 */
+			sigaction_t sa, osa;
 #ifdef HAVE_TCSETPGRP
 			pid_t saved_pgrp = (pid_t)-1;
+			int signo = WSTOPSIG(status);
 			int fd = open(_PATH_TTY, O_RDWR|O_NOCTTY, 0);
-			if (fd != -1)
-			    saved_pgrp = tcgetpgrp(fd);
+			if (fd != -1) {
+			    if ((saved_pgrp = tcgetpgrp(fd)) == ppgrp)
+				saved_pgrp = -1;
+			}
 #endif /* HAVE_TCSETPGRP */
-			if (kill(getpid(), WSTOPSIG(status)) != 0)
-			    warning("kill(%d, %d)", getpid(), WSTOPSIG(status));
+			if (signo == SIGTSTP) {
+			    zero_bytes(&sa, sizeof(sa));
+			    sigemptyset(&sa.sa_mask);
+			    sa.sa_handler = SIG_DFL;
+			    sigaction(SIGTSTP, &sa, &osa);
+			}
+			if (kill(getpid(), signo) != 0)
+			    warning("kill(%d, %d)", getpid(), signo);
+			if (signo == SIGTSTP)
+			    sigaction(SIGTSTP, &osa, NULL);
 #ifdef HAVE_TCSETPGRP
 			if (fd != -1) {
+			    /*
+			     * Restore command's process group if different.
+			     * Otherwise, we cannot resume some shells.
+			     */
 			    if (saved_pgrp != (pid_t)-1)
 				(void)tcsetpgrp(fd, saved_pgrp);
 			    close(fd);
@@ -619,9 +695,9 @@ schedule_signal(signo)
 {
     struct sigforward *sigfwd;
 
-    sigfwd = emalloc(sizeof(*sigfwd));
+    sigfwd = ecalloc(1, sizeof(*sigfwd));
     sigfwd->prev = sigfwd;
-    sigfwd->next = NULL;
+    /* sigfwd->next = NULL; */
     sigfwd->signo = signo;
     tq_append(&sigfwd_list, sigfwd);
 }
@@ -631,6 +707,31 @@ schedule_signal(signo)
  * Generic handler for signals passed from parent -> child.
  * The other end of signal_pipe is checked in the main event loop.
  */
+#ifdef SA_SIGINFO
+RETSIGTYPE
+handler(s, info, context)
+    int s;
+    siginfo_t *info;
+    void *context;
+{
+    unsigned char signo = (unsigned char)s;
+
+    /*
+     * If the signal came from the command we ran, just ignore
+     * it since we don't want the child to indirectly kill itself.
+     * This can happen with, e.g. BSD-derived versions of reboot
+     * that call kill(-1, SIGTERM) to kill all other processes.
+     */
+    if (info != NULL && info->si_code == SI_USER && info->si_pid == cmnd_pid)
+	return;
+
+    /*
+     * The pipe is non-blocking, if we overflow the kernel's pipe
+     * buffer we drop the signal.  This is not a problem in practice.
+     */
+    ignore_result(write(signal_pipe[1], &signo, sizeof(signo)));
+}
+#else
 RETSIGTYPE
 handler(s)
     int s;
@@ -641,9 +742,9 @@ handler(s)
      * The pipe is non-blocking, if we overflow the kernel's pipe
      * buffer we drop the signal.  This is not a problem in practice.
      */
-    if (write(signal_pipe[1], &signo, sizeof(signo)) == -1)
-	/* shut up glibc */;
+    ignore_result(write(signal_pipe[1], &signo, sizeof(signo)));
 }
+#endif
 
 #ifdef SA_SIGINFO
 /*
@@ -653,7 +754,7 @@ handler(s)
  * signals that are generated by the kernel.
  */
 static RETSIGTYPE
-handler_nofwd(s, info, context)
+handler_user_only(s, info, context)
     int s;
     siginfo_t *info;
     void *context;
@@ -661,13 +762,12 @@ handler_nofwd(s, info, context)
     unsigned char signo = (unsigned char)s;
 
     /* Only forward user-generated signals. */
-    if (info->si_code <= 0) {
+    if (info != NULL && info->si_code == SI_USER) {
 	/*
 	 * The pipe is non-blocking, if we overflow the kernel's pipe
 	 * buffer we drop the signal.  This is not a problem in practice.
 	 */
-	if (write(signal_pipe[1], &signo, sizeof(signo)) == -1)
-	    /* shut up glibc */;
+	ignore_result(write(signal_pipe[1], &signo, sizeof(signo)));
     }
 }
 #endif /* SA_SIGINFO */
