@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2009-2010 Todd C. Miller <Todd.Miller@courtesan.com>
+ * Copyright (c) 2009-2012 Todd C. Miller <Todd.Miller@courtesan.com>
  *
  * Permission to use, copy, modify, and distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
@@ -97,7 +97,7 @@ static int io_fds[6] = { -1, -1, -1, -1, -1, -1};
 static int pipeline = FALSE;
 static int tty_initialized;
 static int ttymode = TERM_COOKED;
-static pid_t ppgrp, child, child_pgrp;
+static pid_t ppgrp, cmnd_pgrp;
 static struct io_buffer *iobufs;
 
 static void flush_output __P((void));
@@ -146,9 +146,52 @@ check_foreground()
     }
 }
 
+/*
+ * Generic handler for signals recieved by the monitor process.
+ * The other end of signal_pipe is checked in the monitor event loop.
+ */
+#ifdef SA_SIGINFO
+void
+mon_handler(s, info, context)
+    int s;
+    siginfo_t *info;
+    void *context;
+{
+    unsigned char signo = (unsigned char)s;
+
+    /*
+     * If the signal came from the command we ran, just ignore
+     * it since we don't want the command to indirectly kill itself.
+     * This can happen with, e.g. BSD-derived versions of reboot
+     * that call kill(-1, SIGTERM) to kill all other processes.
+     */
+    if (info != NULL && info->si_code == SI_USER && info->si_pid == cmnd_pid)
+	    return;
+
+    /*
+     * The pipe is non-blocking, if we overflow the kernel's pipe
+     * buffer we drop the signal.  This is not a problem in practice.
+     */
+    ignore_result(write(signal_pipe[1], &signo, sizeof(signo)));
+}
+#else
+void
+mon_handler(s)
+    int s;
+{
+    unsigned char signo = (unsigned char)s;
+
+    /*
+     * The pipe is non-blocking, if we overflow the kernel's pipe
+     * buffer we drop the signal.  This is not a problem in practice.
+     */
+    ignore_result(write(signal_pipe[1], &signo, sizeof(signo)));
+}
+#endif
+
 /*
  * Suspend sudo if the underlying command is suspended.
- * Returns SIGCONT_FG if the child should be resume in the
+ * Returns SIGCONT_FG if the command should be resume in the
  * foreground or SIGCONT_BG if it is a background process.
  */
 int
@@ -162,7 +205,7 @@ suspend_parent(signo)
     case SIGTTOU:
     case SIGTTIN:
 	/*
-	 * If we are the foreground process, just resume the child.
+	 * If we are the foreground process, just resume the command.
 	 * Otherwise, re-send the signal with the handler disabled.
 	 */
 	if (!foreground)
@@ -174,7 +217,7 @@ suspend_parent(signo)
 		} while (!n && errno == EINTR);
 		ttymode = TERM_RAW;
 	    }
-	    rval = SIGCONT_FG; /* resume child in foreground */
+	    rval = SIGCONT_FG; /* resume command in foreground */
 	    break;
 	}
 	ttymode = TERM_RAW;
@@ -191,7 +234,10 @@ suspend_parent(signo)
 	    } while (!n && errno == EINTR);
 	}
 
-	/* Suspend self and continue child when we resume. */
+	/* Suspend self and continue command when we resume. */
+	zero_bytes(&sa, sizeof(sa));
+	sigemptyset(&sa.sa_mask);
+	sa.sa_flags = SA_INTERRUPT; /* do not restart syscalls */
 	sa.sa_handler = SIG_DFL;
 	sigaction(signo, &sa, &osa);
 	if (killpg(ppgrp, signo) != 0)
@@ -202,7 +248,7 @@ suspend_parent(signo)
 
 	/*
 	 * Only modify term if we are foreground process and either
-	 * the old tty mode was not cooked or child got SIGTT{IN,OU}
+	 * the old tty mode was not cooked or command got SIGTT{IN,OU}
 	 */
 	if (ttymode != TERM_COOKED) {
 	    if (foreground) {
@@ -225,10 +271,10 @@ suspend_parent(signo)
 }
 
 /*
- * Kill child with increasing urgency.
+ * Kill command with increasing urgency.
  */
 static void
-terminate_child(pid, use_pgrp)
+terminate_command(pid, use_pgrp)
     pid_t pid;
     int use_pgrp;
 {
@@ -261,8 +307,7 @@ io_buf_new(rfd, wfd, action, head)
 {
     struct io_buffer *iob;
 
-    iob = emalloc(sizeof(*iob));
-    zero_bytes(iob, sizeof(*iob));
+    iob = ecalloc(1, sizeof(*iob));
     iob->rfd = rfd;
     iob->wfd = wfd;
     iob->action = action;
@@ -292,13 +337,12 @@ perform_io(fdsr, fdsw, cstat)
 	    } while (n == -1 && errno == EINTR);
 	    switch (n) {
 		case -1:
-		    if (errno == EAGAIN)
-			break;
-		    if (errno != ENXIO && errno != EBADF) {
-			errors++;
-			break;
+		    if (errno != EAGAIN) {
+			/* treat read error as fatal and close the fd */
+			safe_close(iob->rfd);
+			iob->rfd = -1;
 		    }
-		    /* FALLTHROUGH */
+		    break;
 		case 0:
 		    /* got EOF or pty has gone away */
 		    safe_close(iob->rfd);
@@ -306,7 +350,7 @@ perform_io(fdsr, fdsw, cstat)
 		    break;
 		default:
 		    if (!iob->action(iob->buf + iob->len, n))
-			terminate_child(child, TRUE);
+			terminate_command(cmnd_pid, TRUE);
 		    iob->len += n;
 		    break;
 	    }
@@ -317,7 +361,7 @@ perform_io(fdsr, fdsw, cstat)
 		    iob->len - iob->off);
 	    } while (n == -1 && errno == EINTR);
 	    if (n == -1) {
-		if (errno == EPIPE || errno == ENXIO || errno == EBADF) {
+		if (errno == EPIPE || errno == ENXIO || errno == EIO || errno == EBADF) {
 		    /* other end of pipe closed or pty revoked */
 		    if (iob->rfd != -1) {
 			safe_close(iob->rfd);
@@ -347,7 +391,7 @@ perform_io(fdsr, fdsw, cstat)
  * Returns the child pid.
  */
 int
-fork_pty(path, argv, envp, sv, rbac_enabled, bgmode, maxfd)
+fork_pty(path, argv, envp, sv, rbac_enabled, bgmode, maxfd, omask)
     const char *path;
     char *argv[];
     char *envp[];
@@ -355,11 +399,14 @@ fork_pty(path, argv, envp, sv, rbac_enabled, bgmode, maxfd)
     int rbac_enabled;
     int bgmode;
     int *maxfd;
+    sigset_t *omask;
 {
     struct command_status cstat;
     struct io_buffer *iob;
     int io_pipe[3][2], n;
     sigaction_t sa;
+    sigset_t mask;
+    pid_t child;
 
     ppgrp = getpgrp(); /* parent's pgrp, so child can signal us */
 
@@ -427,9 +474,19 @@ fork_pty(path, argv, envp, sv, rbac_enabled, bgmode, maxfd)
 
     /* Job control signals to relay from parent to child. */
     sa.sa_flags = SA_INTERRUPT; /* do not restart syscalls */
+#ifdef SA_SIGINFO
+    sa.sa_flags |= SA_SIGINFO;
+    sa.sa_sigaction = handler;
+#else
     sa.sa_handler = handler;
+#endif
     sigaction(SIGTSTP, &sa, NULL);
 
+    /* We don't want to receive SIGTTIN/SIGTTOU, getting EIO is preferable. */
+    sa.sa_handler = SIG_IGN;
+    sigaction(SIGTTIN, &sa, NULL);
+    sigaction(SIGTTOU, &sa, NULL);
+
     if (foreground) {
 	/* Copy terminal attrs from user tty -> pty slave. */
 	if (term_copy(io_fds[SFD_USERTTY], io_fds[SFD_SLAVE])) {
@@ -448,6 +505,17 @@ fork_pty(path, argv, envp, sv, rbac_enabled, bgmode, maxfd)
 	}
     }
 
+    /*
+     * Block some signals until cmnd_pid is set in the parent to avoid a
+     * race between exec of the command and receipt of a fatal signal from it.
+     */
+    sigemptyset(&mask);
+    sigaddset(&mask, SIGTERM);
+    sigaddset(&mask, SIGHUP);
+    sigaddset(&mask, SIGINT);
+    sigaddset(&mask, SIGQUIT);
+    sigprocmask(SIG_BLOCK, &mask, omask);
+
     child = fork();
     switch (child) {
     case -1:
@@ -459,6 +527,7 @@ fork_pty(path, argv, envp, sv, rbac_enabled, bgmode, maxfd)
 	close(signal_pipe[0]);
 	close(signal_pipe[1]);
 	fcntl(sv[1], F_SETFD, FD_CLOEXEC);
+	sigprocmask(SIG_SETMASK, omask, NULL);
 	if (exec_setup(rbac_enabled, slavename, io_fds[SFD_SLAVE]) == TRUE) {
 	    /* Close the other end of the stdin/stdout/stderr pipes and exec. */
 	    if (io_pipe[STDIN_FILENO][1])
@@ -471,7 +540,7 @@ fork_pty(path, argv, envp, sv, rbac_enabled, bgmode, maxfd)
 	}
 	cstat.type = CMD_ERRNO;
 	cstat.val = errno;
-	send(sv[1], &cstat, sizeof(cstat), 0);
+	ignore_result(send(sv[1], &cstat, sizeof(cstat), 0));
 	_exit(1);
     }
 
@@ -523,9 +592,12 @@ pty_close(cstat)
     flush_output();
 
     if (io_fds[SFD_USERTTY] != -1) {
-	do {
-	    n = term_restore(io_fds[SFD_USERTTY], 0);
-	} while (!n && errno == EINTR);
+	check_foreground();
+	if (foreground) {
+	    do {
+		n = term_restore(io_fds[SFD_USERTTY], 0);
+	    } while (!n && errno == EINTR);
+	}
     }
 
     /* If child was signalled, write the reason to stdout like the shell. */
@@ -537,11 +609,9 @@ pty_close(cstat)
 		io_fds[SFD_USERTTY] : STDOUT_FILENO;
 	    if (write(n, reason, strlen(reason)) != -1) {
 		if (WCOREDUMP(cstat->val)) {
-		    if (write(n, " (core dumped)", 14) == -1)
-			/* shut up glibc */;
+		    ignore_result(write(n, " (core dumped)", 14));
 		}
-		if (write(n, "\n", 1) == -1)
-		    /* shut up glibc */;
+		ignore_result(write(n, "\n", 1));
 	    }
 	}
     }
@@ -599,7 +669,7 @@ deliver_signal(pid, signo)
     case SIGCONT_FG:
 	/* Continue in foreground, grant it controlling tty. */
 	do {
-	    status = tcsetpgrp(io_fds[SFD_SLAVE], child_pgrp);
+	    status = tcsetpgrp(io_fds[SFD_SLAVE], cmnd_pgrp);
 	} while (status == -1 && errno == EINTR);
 	killpg(pid, SIGCONT);
 	break;
@@ -614,7 +684,7 @@ deliver_signal(pid, signo)
 	_exit(1); /* XXX */
 	/* NOTREACHED */
     default:
-	/* Relay signal to child. */
+	/* Relay signal to command. */
 	killpg(pid, signo);
 	break;
     }
@@ -641,10 +711,10 @@ send_status(fd, cstat)
 }
 
 /*
- * Wait for child status after receiving SIGCHLD.
- * If the child was stopped, the status is send back to the parent.
+ * Wait for command status after receiving SIGCHLD.
+ * If the command was stopped, the status is send back to the parent.
  * Otherwise, cstat is filled in but not sent.
- * Returns TRUE if child is still alive, else FALSE.
+ * Returns TRUE if command is still alive, else FALSE.
  */
 static int
 handle_sigchld(backchannel, cstat)
@@ -654,22 +724,22 @@ handle_sigchld(backchannel, cstat)
     int status, alive = TRUE;
     pid_t pid;
 
-    /* read child status */
+    /* read command status */
     do {
 #ifdef sudo_waitpid
-	pid = sudo_waitpid(child, &status, WUNTRACED|WNOHANG);
+	pid = sudo_waitpid(cmnd_pid, &status, WUNTRACED|WNOHANG);
 #else
 	pid = wait(&status);
 #endif
     } while (pid == -1 && errno == EINTR);
-    if (pid == child) {
+    if (pid == cmnd_pid) {
 	if (cstat->type != CMD_ERRNO) {
 	    cstat->type = CMD_WSTATUS;
 	    cstat->val = status;
 	    if (WIFSTOPPED(status)) {
 		do {
-		    child_pgrp = tcgetpgrp(io_fds[SFD_SLAVE]);
-		} while (child_pgrp == -1 && errno == EINTR);
+		    cmnd_pgrp = tcgetpgrp(io_fds[SFD_SLAVE]);
+		} while (cmnd_pgrp == -1 && errno == EINTR);
 		if (send_status(backchannel, cstat) == -1)
 		    return alive; /* XXX */
 	    }
@@ -730,12 +800,22 @@ exec_monitor(path, argv, envp, backchannel, rbac)
 
     /* Note: HP-UX select() will not be interrupted if SA_RESTART set */
     sa.sa_flags = SA_INTERRUPT;
-    sa.sa_handler = handler;
+#ifdef SA_SIGINFO
+    sa.sa_flags |= SA_SIGINFO;
+    sa.sa_sigaction = mon_handler;
+#else
+    sa.sa_handler = mon_handler;
+#endif
     sigaction(SIGCHLD, &sa, NULL);
 
     /* Catch common signals so we can cleanup properly. */
     sa.sa_flags = SA_RESTART;
-    sa.sa_handler = handler;
+#ifdef SA_SIGINFO
+    sa.sa_flags |= SA_SIGINFO;
+    sa.sa_sigaction = mon_handler;
+#else
+    sa.sa_handler = mon_handler;
+#endif
     sigaction(SIGHUP, &sa, NULL);
     sigaction(SIGINT, &sa, NULL);
     sigaction(SIGQUIT, &sa, NULL);
@@ -747,7 +827,7 @@ exec_monitor(path, argv, envp, backchannel, rbac)
     /*
      * Start a new session with the parent as the session leader
      * and the slave pty as the controlling terminal.
-     * This allows us to be notified when the child has been suspended.
+     * This allows us to be notified when the command has been suspended.
      */
     if (setsid() == -1) {
 	warning("setsid");
@@ -776,12 +856,12 @@ exec_monitor(path, argv, envp, backchannel, rbac)
     /* Start command and wait for it to stop or exit */
     if (pipe(errpipe) == -1)
 	error(1, "unable to create pipe");
-    child = fork();
-    if (child == -1) {
+    cmnd_pid = fork();
+    if (cmnd_pid == -1) {
 	warning("Can't fork");
 	goto bad;
     }
-    if (child == 0) {
+    if (cmnd_pid == 0) {
 	/* We pass errno back to our parent via pipe on exec failure. */
 	close(backchannel);
 	close(signal_pipe[0]);
@@ -794,12 +874,16 @@ exec_monitor(path, argv, envp, backchannel, rbac)
 	exec_pty(path, argv, envp, rbac, &errpipe[1]);
 	cstat.type = CMD_ERRNO;
 	cstat.val = errno;
-	if (write(errpipe[1], &cstat, sizeof(cstat)) == -1)
-	    /* shut up glibc */;
+	ignore_result(write(errpipe[1], &cstat, sizeof(cstat)));
 	_exit(1);
     }
     close(errpipe[1]);
 
+    /* Send the command's pid to main sudo process. */
+    cstat.type = CMD_PID;
+    cstat.val = cmnd_pid;
+    ignore_result(send(backchannel, &cstat, sizeof(cstat), 0));
+
     /* If any of stdin/stdout/stderr are pipes, close them in parent. */
     if (io_fds[SFD_STDIN] != io_fds[SFD_SLAVE])
 	close(io_fds[SFD_STDIN]);
@@ -809,22 +893,20 @@ exec_monitor(path, argv, envp, backchannel, rbac)
 	close(io_fds[SFD_STDERR]);
 
     /*
-     * Put child in its own process group.  If we are starting the command
+     * Put command in its own process group.  If we are starting the command
      * in the foreground, assign its pgrp to the tty.
      */
-    child_pgrp = child;
-    setpgid(child, child_pgrp);
+    cmnd_pgrp = cmnd_pid;
+    setpgid(cmnd_pid, cmnd_pgrp);
     if (foreground) {
 	do {
-	    status = tcsetpgrp(io_fds[SFD_SLAVE], child_pgrp);
+	    status = tcsetpgrp(io_fds[SFD_SLAVE], cmnd_pgrp);
 	} while (status == -1 && errno == EINTR);
     }
 
     /* Wait for errno on pipe, signal on backchannel or for SIGCHLD */
     maxfd = MAX(MAX(errpipe[0], signal_pipe[0]), backchannel);
-    fdsr = (fd_set *)emalloc2(howmany(maxfd + 1, NFDBITS), sizeof(fd_mask));
-    zero_bytes(fdsr, howmany(maxfd + 1, NFDBITS) * sizeof(fd_mask));
-    zero_bytes(&cstat, sizeof(cstat));
+    fdsr = ecalloc(howmany(maxfd + 1, NFDBITS), sizeof(fd_mask));
     tv.tv_sec = 0;
     tv.tv_usec = 0;
     for (;;) {
@@ -840,9 +922,10 @@ exec_monitor(path, argv, envp, backchannel, rbac)
 	if (n <= 0) {
 	    if (n == 0)
 		goto done;
-	    if (errno == EINTR)
+	    if (errno == EINTR || errno == ENOMEM)
 		continue;
-	    error(1, "select failed");
+	    warning("monitor: select failed");
+	    break;
 	}
 
 	if (FD_ISSET(signal_pipe[0], fdsr)) {
@@ -855,12 +938,14 @@ exec_monitor(path, argv, envp, backchannel, rbac)
 	    }
 	    /*
 	     * Handle SIGCHLD specially and deliver other signals
-	     * directly to the child.
+	     * directly to the command.
 	     */
-	    if (signo == SIGCHLD)
-		alive = handle_sigchld(backchannel, &cstat);
-	    else
-		deliver_signal(child, signo);
+	    if (signo == SIGCHLD) {
+		if (!handle_sigchld(backchannel, &cstat))
+		    alive = FALSE;
+	    } else {
+		deliver_signal(cmnd_pid, signo);
+	    }
 	    continue;
 	}
 	if (errpipe[0] != -1 && FD_ISSET(errpipe[0], fdsr)) {
@@ -892,14 +977,14 @@ exec_monitor(path, argv, envp, backchannel, rbac)
 		warningx("unexpected reply type on backchannel: %d", cstmp.type);
 		continue;
 	    }
-	    deliver_signal(child, cstmp.val);
+	    deliver_signal(cmnd_pid, cstmp.val);
 	}
     }
 
 done:
     if (alive) {
 	/* XXX An error occurred, should send an error back. */
-	kill(child, SIGKILL);
+	kill(cmnd_pid, SIGKILL);
     } else {
 	/* Send parent status. */
 	send_status(backchannel, &cstat);
@@ -932,11 +1017,11 @@ flush_output()
     if (maxfd == -1)
 	return;
 
-    fdsr = (fd_set *)emalloc2(howmany(maxfd + 1, NFDBITS), sizeof(fd_mask));
-    fdsw = (fd_set *)emalloc2(howmany(maxfd + 1, NFDBITS), sizeof(fd_mask));
+    fdsr = emalloc2(howmany(maxfd + 1, NFDBITS), sizeof(fd_mask));
+    fdsw = emalloc2(howmany(maxfd + 1, NFDBITS), sizeof(fd_mask));
     for (;;) {
-	zero_bytes(fdsw, howmany(maxfd + 1, NFDBITS) * sizeof(fd_mask));
-	zero_bytes(fdsr, howmany(maxfd + 1, NFDBITS) * sizeof(fd_mask));
+	memset(fdsw, 0, howmany(maxfd + 1, NFDBITS) * sizeof(fd_mask));
+	memset(fdsr, 0, howmany(maxfd + 1, NFDBITS) * sizeof(fd_mask));
 
 	nwriters = 0;
 	for (iob = iobufs; iob; iob = iob->next) {
@@ -972,11 +1057,11 @@ flush_output()
 	if (nready <= 0) {
 	    if (nready == 0)
 		break; /* all I/O flushed */
-	    if (errno == EINTR)
+	    if (errno == EINTR || errno == ENOMEM)
 		continue;
-	    error(1, "select failed");
+	    warning("select failed");
 	}
-	if (perform_io(fdsr, fdsw, NULL) != 0)
+	if (perform_io(fdsr, fdsw, NULL) != 0 || nready == -1)
 	    break;
     }
     efree(fdsr);
@@ -998,7 +1083,7 @@ exec_pty(path, argv, envp, rbac_enabled, errfd)
     int maxfd = def_closefrom;
     pid_t self = getpid();
 
-    /* Set child process group here too to avoid a race. */
+    /* Set command process group here too to avoid a race. */
     setpgid(0, self);
 
     /* Wire up standard fds, note that stdout/stderr may be pipes. */
@@ -1083,3 +1168,14 @@ safe_close(fd)
     }
     return close(fd);
 }
+
+void
+cleanup_pty(gotsignal)
+    int gotsignal;
+{
+    if (io_fds[SFD_USERTTY] != -1) {
+	check_foreground();
+	if (foreground)
+	    term_restore(io_fds[SFD_USERTTY], 0);
+    }
+}
