@@ -99,7 +99,7 @@ load_interfaces()
 {
     struct ifaddrs *ifa, *ifaddrs;
     struct sockaddr_in *sin;
-#ifdef HAVE_IN6_ADDR
+#ifdef HAVE_STRUCT_IN6_ADDR
     struct sockaddr_in6 *sin6;
 #endif
     int i;
@@ -116,7 +116,7 @@ load_interfaces()
 
 	switch(ifa->ifa_addr->sa_family) {
 	    case AF_INET:
-#ifdef HAVE_IN6_ADDR
+#ifdef HAVE_STRUCT_IN6_ADDR
 	    case AF_INET6:
 #endif
 		num_interfaces++;
@@ -150,7 +150,7 @@ load_interfaces()
 		interfaces[i].family = AF_INET;
 		i++;
 		break;
-#ifdef HAVE_IN6_ADDR
+#ifdef HAVE_STRUCT_IN6_ADDR
 	    case AF_INET6:
 		sin6 = (struct sockaddr_in6 *)ifa->ifa_addr;
 		if (sin6 == NULL)
@@ -165,7 +165,7 @@ load_interfaces()
 		interfaces[i].family = AF_INET6;
 		i++;
 		break;
-#endif /* HAVE_IN6_ADDR */
+#endif /* HAVE_STRUCT_IN6_ADDR */
 	}
     }
 #ifdef HAVE_FREEIFADDRS
@@ -328,7 +328,7 @@ void
 dump_interfaces()
 {
     int i;
-#ifdef HAVE_IN6_ADDR
+#ifdef HAVE_STRUCT_IN6_ADDR
     char addrbuf[INET6_ADDRSTRLEN], maskbuf[INET6_ADDRSTRLEN];
 #endif
 
@@ -339,7 +339,7 @@ dump_interfaces()
 		printf("\t%s / ", inet_ntoa(interfaces[i].addr.ip4));
 		puts(inet_ntoa(interfaces[i].netmask.ip4));
 		break;
-#ifdef HAVE_IN6_ADDR
+#ifdef HAVE_STRUCT_IN6_ADDR
 	    case AF_INET6:
 		inet_ntop(AF_INET6, &interfaces[i].addr.ip6,
 		    addrbuf, sizeof(addrbuf));
@@ -347,7 +347,7 @@ dump_interfaces()
 		    maskbuf, sizeof(maskbuf));
 		printf("\t%s / %s\n", addrbuf, maskbuf);
 		break;
-#endif /* HAVE_IN6_ADDR */
+#endif /* HAVE_STRUCT_IN6_ADDR */
 	}
     }
 }
