@@ -33,8 +33,8 @@ struct lbuf {
 };
 
 int get_ttycols		__P((void));
-void lbuf_append	__P((struct lbuf *, ...));
-void lbuf_append_quoted	__P((struct lbuf *, const char *, ...));
+void lbuf_append	__P((struct lbuf *, const char *, ...) __printflike(2, 3));
+void lbuf_append_quoted	__P((struct lbuf *, const char *, const char *, ...)) __printflike(3, 4);
 void lbuf_destroy	__P((struct lbuf *));
 void lbuf_init		__P((struct lbuf *, int (*)(const char *), int, const char *));
 void lbuf_print		__P((struct lbuf *));
