@@ -47,15 +47,6 @@
 # define __attribute__(x)
 #endif
 
-/* For silencing gcc warnings about rcsids */
-#ifndef __unused
-# if defined(__GNUC__) && (__GNUC__ > 2 || __GNUC__ == 2 && __GNUC_MINOR__ > 7)
-#  define __unused	__attribute__((__unused__))
-# else
-#  define __unused
-# endif
-#endif
-
 /* For catching format string mismatches */
 #ifndef __printflike
 # if defined(__GNUC__) && (__GNUC__ > 2 || __GNUC__ == 2 && __GNUC_MINOR__ >= 7)
@@ -157,6 +148,17 @@
 # define MAX(a,b) (((a)>(b))?(a):(b))
 #endif
 
+/*
+ * Older systems may be missing stddef.h and/or offsetof macro
+ */
+#ifndef offsetof
+# ifdef __offsetof
+#  define offsetof(type, field) __offsetof(type, field)
+# else
+#  define offsetof(type, field) ((size_t)(&((type *)0)->field))
+# endif
+#endif
+
 /*
  * Simple isblank() macro and function for systems without it.
  */
