@@ -37,7 +37,7 @@ main(argc, argv)
     int argc;
     char *argv[];
 {
-    static char *my_sys_siglist[NSIG];
+    static char *sudo_sys_siglist[NSIG];
     int i;
 
 #include "mksiglist.h"
@@ -45,10 +45,10 @@ main(argc, argv)
     printf("#include <config.h>\n");
     printf("#include <signal.h>\n");
     printf("#include \"missing.h\"\n\n");
-    printf("const char *const my_sys_siglist[NSIG] = {\n");
+    printf("const char *const sudo_sys_siglist[NSIG] = {\n");
     for (i = 0; i < NSIG; i++) {
-	if (my_sys_siglist[i] != NULL) {
-	    printf("    \"%s\",\n", my_sys_siglist[i]);
+	if (sudo_sys_siglist[i] != NULL) {
+	    printf("    \"%s\",\n", sudo_sys_siglist[i]);
 	} else {
 	    printf("    \"Signal %d\",\n", i);
 	}
