@@ -116,10 +116,10 @@ sudo_file_parse(nss)
     yyin = nss->handle;
     if (yyparse() != 0 || parse_error) {
 	if (errorlineno != -1) {
-	    log_error(NO_EXIT, "parse error in %s near line %d",
+	    log_error(0, "parse error in %s near line %d",
 		errorfile, errorlineno);
 	} else {
-	    log_error(NO_EXIT, "parse error in %s", errorfile);
+	    log_error(0, "parse error in %s", errorfile);
 	}
 	return -1;
     }
@@ -480,10 +480,9 @@ sudo_file_display_defaults(nss, pw, lbuf)
 	    case DEFAULTS_CMND:
 		continue;
 	}
-	lbuf_append(lbuf, prefix);
 	if (d->val != NULL) {
-	    lbuf_append(lbuf, "%s%s", d->var, d->op == '+' ? "+=" :
-		d->op == '-' ? "-=" : "=");
+	    lbuf_append(lbuf, "%s%s%s", prefix, d->var,
+		d->op == '+' ? "+=" : d->op == '-' ? "-=" : "=");
 	    if (strpbrk(d->val, " \t") != NULL) {
 		lbuf_append(lbuf, "\"");
 		lbuf_append_quoted(lbuf, "\"", "%s", d->val);
@@ -491,7 +490,8 @@ sudo_file_display_defaults(nss, pw, lbuf)
 	    } else
 		lbuf_append_quoted(lbuf, SUDOERS_QUOTED, "%s", d->val);
 	} else
-	    lbuf_append(lbuf, "%s%s", d->op == FALSE ? "!" : "", d->var);
+	    lbuf_append(lbuf, "%s%s%s", prefix,
+		d->op == FALSE ? "!" : "", d->var);
 	prefix = ", ";
 	nfound++;
     }
