@@ -141,8 +141,10 @@ set_perms(perm)
 
 	case PERM_SUDOERS:
 				/* assume euid == ROOT_UID, ruid == user */
-				if (setresgid(-1, SUDOERS_GID, -1))
-				    error(1, "unable to change to sudoers gid");
+				if (setresgid(-1, SUDOERS_GID, -1)) {
+				    errstr = "unable to change to sudoers gid";
+				    goto bad;
+				}
 
 				/*
 				 * If SUDOERS_UID == ROOT_UID and SUDOERS_MODE
@@ -152,7 +154,7 @@ set_perms(perm)
 				 * work on all OS's.
 				 */
 				if (SUDOERS_UID == ROOT_UID) {
-				    if ((SUDOERS_MODE & 040) && setresuid(ROOT_UID, 1, ROOT_UID)) {
+				    if ((SUDOERS_MODE & S_IRGRP) && setresuid(ROOT_UID, 1, ROOT_UID)) {
 					errstr = "setresuid(ROOT_UID, 1, ROOT_UID)";
 					goto bad;
 				    }
@@ -258,8 +260,10 @@ set_perms(perm)
 
 	case PERM_SUDOERS:
 				/* assume euid == ROOT_UID, ruid == user */
-				if (setregid(-1, SUDOERS_GID))
-				    error(1, "unable to change to sudoers gid");
+				if (setregid(-1, SUDOERS_GID)) {
+				    errstr = "unable to change to sudoers gid";
+				    goto bad;
+				}
 
 				/*
 				 * If SUDOERS_UID == ROOT_UID and SUDOERS_MODE
@@ -269,7 +273,7 @@ set_perms(perm)
 				 * work on all OS's.
 				 */
 				if (SUDOERS_UID == ROOT_UID) {
-				    if ((SUDOERS_MODE & 040) && setreuid(ROOT_UID, 1)) {
+				    if ((SUDOERS_MODE & S_IRGRP) && setreuid(ROOT_UID, 1)) {
 					errstr = "setreuid(ROOT_UID, 1)";
 					goto bad;
 				    }
@@ -377,8 +381,10 @@ set_perms(perm)
 				break;
 
 	case PERM_SUDOERS:
-				if (setegid(SUDOERS_GID))
-				    error(1, "unable to change to sudoers gid");
+				if (setegid(SUDOERS_GID)) {
+				    errstr = "unable to change to sudoers gid";
+				    goto bad;
+				}
 
 				/*
 				 * If SUDOERS_UID == ROOT_UID and SUDOERS_MODE
@@ -388,7 +394,7 @@ set_perms(perm)
 				 * work on all OS's.
 				 */
 				if (SUDOERS_UID == ROOT_UID) {
-				    if ((SUDOERS_MODE & 040) && seteuid(1)) {
+				    if ((SUDOERS_MODE & S_IRGRP) && seteuid(1)) {
 					errstr = "seteuid(1)";
 					goto bad;
 				    }
@@ -507,7 +513,7 @@ runas_setgroups()
 	aix_setauthdb(pw->pw_name);
 # endif
 	if (initgroups(pw->pw_name, pw->pw_gid) < 0)
-	    log_error(USE_ERRNO|MSG_ONLY, "can't set runas group vector");
+	    log_fatal(USE_ERRNO|MSG_ONLY, "can't set runas group vector");
 # ifdef HAVE_GETGROUPS
 	if (groups) {
 	    efree(groups);
@@ -516,14 +522,14 @@ runas_setgroups()
 	if ((ngroups = getgroups(0, NULL)) > 0) {
 	    groups = emalloc2(ngroups, sizeof(GETGROUPS_T));
 	    if (getgroups(ngroups, groups) < 0)
-		log_error(USE_ERRNO|MSG_ONLY, "can't get runas group vector");
+		log_fatal(USE_ERRNO|MSG_ONLY, "can't get runas group vector");
 	}
 #  ifdef HAVE_SETAUTHDB
 	aix_restoreauthdb();
 #  endif
     } else {
 	if (setgroups(ngroups, groups) < 0)
-	    log_error(USE_ERRNO|MSG_ONLY, "can't set runas group vector");
+	    log_fatal(USE_ERRNO|MSG_ONLY, "can't set runas group vector");
 # endif /* HAVE_GETGROUPS */
     }
 }
@@ -532,7 +538,7 @@ static void
 restore_groups()
 {
     if (user_ngroups >= 0 && setgroups(user_ngroups, user_groups) < 0)
-	log_error(USE_ERRNO|MSG_ONLY, "can't reset user group vector");
+	log_fatal(USE_ERRNO|MSG_ONLY, "can't reset user group vector");
 }
 
 #else
