@@ -87,6 +87,9 @@
 # ifndef LOGIN_DEFROOTCLASS
 #  define LOGIN_DEFROOTCLASS	"daemon"
 # endif
+# ifndef LOGIN_SETENV
+#  define LOGIN_SETENV	0
+# endif
 #endif
 #ifdef HAVE_MBR_CHECK_MEMBERSHIP
 # include <membership.h>
@@ -103,6 +106,7 @@
 #include "sudo.h"
 #include "lbuf.h"
 #include "interfaces.h"
+#include "secure_path.h"
 #include <sudo_usage.h>
 
 #ifdef USING_NONUNIX_GROUPS
@@ -129,6 +133,7 @@ static void show_version		__P((void));
 static void create_admin_success_flag	__P((void));
 extern int sudo_edit			__P((int, char **, char **));
 int run_command __P((const char *path, char *argv[], char *envp[], uid_t uid, int dowait)); /* XXX should be in sudo.h */
+static void sudo_check_suid		__P((const char *path));
 
 /*
  * Globals
@@ -173,7 +178,7 @@ main(argc, argv, envp)
     char *envp[];
 {
     int sources = 0, validated;
-    int fd, cmnd_status, pwflag, rc = 0;
+    int fd, cmnd_status, pwflag, rval = TRUE;
     sigaction_t sa;
     struct sudo_nss *nss;
 #if defined(SUDO_DEVEL) && defined(__OpenBSD__)
@@ -197,8 +202,8 @@ main(argc, argv, envp)
 # endif
 #endif /* HAVE_GETPRPWNAM && HAVE_SET_AUTH_PARAMETERS */
 
-    if (geteuid() != 0)
-	errorx(1, "must be setuid root");
+    /* Make sure we are setuid root. */
+    sudo_check_suid(argv[0]);
 
     /*
      * Signal setup:
@@ -258,7 +263,7 @@ main(argc, argv, envp)
 		break;
 	    case MODE_LISTDEFS:
 		list_options();
-		exit(0);
+		goto done;
 		break;
 	    case MODE_LIST:
 	    case MODE_LIST|MODE_INVALIDATE:
@@ -290,11 +295,11 @@ main(argc, argv, envp)
 	if (nss->open(nss) == 0 && nss->parse(nss) == 0) {
 	    sources++;
 	    if (nss->setdefs(nss) != 0)
-		log_error(NO_STDERR|NO_EXIT, "problem with defaults entries");
+		log_error(NO_STDERR, "problem with defaults entries");
 	}
     }
     if (sources == 0)
-	log_error(0, "no valid sudoers sources found, quitting");
+	log_fatal(0, "no valid sudoers sources found, quitting");
 
     /* XXX - collect post-sudoers parse settings into a function */
 
@@ -311,7 +316,7 @@ main(argc, argv, envp)
 	set_runaspw(runas_user ? runas_user : def_runas_default);
 
     if (!update_defaults(SETDEF_RUNAS))
-	log_error(NO_STDERR|NO_EXIT, "problem with defaults entries");
+	log_error(NO_STDERR, "problem with defaults entries");
 
     if (def_fqdn)
 	set_fqdn();	/* deferred until after sudoers is parsed */
@@ -326,8 +331,7 @@ main(argc, argv, envp)
     /* This goes after sudoers is parsed since it may have timestamp options. */
     if (sudo_mode == MODE_KILL || sudo_mode == MODE_INVALIDATE) {
 	remove_timestamp((sudo_mode == MODE_KILL));
-	cleanup(0);
-	exit(0);
+	goto done;
     }
 
     /* Is root even allowed to run sudo? */
@@ -335,15 +339,16 @@ main(argc, argv, envp)
 	(void) fprintf(stderr,
 	    "Sorry, %s has been configured to not allow root to run it.\n",
 	    getprogname());
-	exit(1);
+	goto bad;
     }
 
     /* Check for -C overriding def_closefrom. */
     if (user_closefrom >= 0 && user_closefrom != def_closefrom) {
-	if (!def_closefrom_override)
-	    errorx(1, "you are not permitted to use the -C option");
-	else
-	    def_closefrom = user_closefrom;
+	if (!def_closefrom_override) {
+	    warningx("you are not permitted to use the -C option");
+	    goto bad;
+	}
+	def_closefrom = user_closefrom;
     }
 
     /* If given the -P option, set the "preserve_groups" flag. */
@@ -365,7 +370,7 @@ main(argc, argv, envp)
 	validated = nss->lookup(nss, validated, pwflag);
 
 	if (ISSET(validated, VALIDATE_OK)) {
-	    /* Handle "= auth" in netsvc.conf */
+	    /* Handle [SUCCESS=return] */
 	    if (nss->ret_if_found)
 		break;
 	} else {
@@ -396,11 +401,14 @@ main(argc, argv, envp)
 	    pw = sudo_getpwuid(atoi(def_timestampowner + 1));
 	else
 	    pw = sudo_getpwnam(def_timestampowner);
-	if (!pw)
+	if (pw != NULL) {
+	    timestamp_uid = pw->pw_uid;
+	    pw_delref(pw);
+	} else {
 	    log_error(0, "timestamp owner (%s): No such user",
 		def_timestampowner);
-	timestamp_uid = pw->pw_uid;
-	pw_delref(pw);
+	    timestamp_uid = ROOT_UID;
+	}
     }
 
     /* If no command line args and "set_home" is not set, error out. */
@@ -411,7 +419,7 @@ main(argc, argv, envp)
     if (def_requiretty) {
 	if ((fd = open(_PATH_TTY, O_RDWR|O_NOCTTY)) == -1) {
 	    audit_failure(NewArgv, "no tty");
-	    log_error(NO_MAIL, "sorry, you must have a tty to run sudo");
+	    log_fatal(NO_MAIL, "sorry, you must have a tty to run sudo");
 	} else
 	    (void) close(fd);
     }
@@ -432,7 +440,12 @@ main(argc, argv, envp)
     rebuild_env(def_noexec);
 
     /* Require a password if sudoers says so.  */
-    check_user(validated, sudo_mode);
+    rval = check_user(validated, sudo_mode);
+    if (rval != TRUE) {
+	if (!ISSET(validated, VALIDATE_OK))
+	    log_denial(validated, FALSE);
+	goto done;
+    }
 
     /* If run as root with SUDO_USER set, set sudo_user.pw to that user. */
     /* XXX - causes confusion when root is not listed in sudoers */
@@ -451,134 +464,130 @@ main(argc, argv, envp)
 	}
     }
 
-    if (ISSET(validated, VALIDATE_OK)) {
-	/* Create Ubuntu-style dot file to indicate sudo was successful. */
-	create_admin_success_flag();
+    /* If the user was not allowed to run the command we are done. */
+    if (!ISSET(validated, VALIDATE_OK)) {
+	log_failure(validated, cmnd_status);
+	goto bad;
+    }
 
-	/* Finally tell the user if the command did not exist. */
-	if (cmnd_status == NOT_FOUND_DOT) {
-	    audit_failure(NewArgv, "command in current directory");
-	    errorx(1, "ignoring `%s' found in '.'\nUse `sudo ./%s' if this is the `%s' you wish to run.", user_cmnd, user_cmnd, user_cmnd);
-	} else if (cmnd_status == NOT_FOUND) {
-	    audit_failure(NewArgv, "%s: command not found", user_cmnd);
-	    errorx(1, "%s: command not found", user_cmnd);
-	}
+    /* Create Ubuntu-style dot file to indicate sudo was successful. */
+    create_admin_success_flag();
 
-	/* If user specified env vars make sure sudoers allows it. */
-	if (ISSET(sudo_mode, MODE_RUN) && !def_setenv) {
-	    if (ISSET(sudo_mode, MODE_PRESERVE_ENV))
-		log_error(NO_MAIL,
-		    "sorry, you are not allowed to preserve the environment");
-	    else
-		validate_env_vars(sudo_user.env_vars);
-	}
+    /* Finally tell the user if the command did not exist. */
+    if (cmnd_status == NOT_FOUND_DOT) {
+	audit_failure(NewArgv, "command in current directory");
+	warningx("ignoring `%s' found in '.'\nUse `sudo ./%s' if this is the `%s' you wish to run.", user_cmnd, user_cmnd, user_cmnd);
+	goto bad;
+    } else if (cmnd_status == NOT_FOUND) {
+	audit_failure(NewArgv, "%s: command not found", user_cmnd);
+	warningx("%s: command not found", user_cmnd);
+	goto bad;
+    }
+
+    /* If user specified env vars make sure sudoers allows it. */
+    if (ISSET(sudo_mode, MODE_RUN) && !def_setenv) {
+	if (ISSET(sudo_mode, MODE_PRESERVE_ENV)) {
+	    warningx("sorry, you are not allowed to preserve the environment");
+	    goto bad;
+	} else
+	    validate_env_vars(sudo_user.env_vars);
+    }
 
 #ifdef _PATH_SUDO_IO_LOGDIR
-	/* Get next session ID so we can log it. */
-	if (ISSET(sudo_mode, (MODE_RUN | MODE_EDIT)) && (def_log_input || def_log_output))
-	    io_nextid();
+    /* Get next session ID so we can log it. */
+    if (ISSET(sudo_mode, (MODE_RUN | MODE_EDIT)) && (def_log_input || def_log_output))
+	io_nextid();
 #endif
-	log_allowed(validated);
-	if (ISSET(sudo_mode, MODE_CHECK))
-	    rc = display_cmnd(snl, list_pw ? list_pw : sudo_user.pw);
-	else if (ISSET(sudo_mode, MODE_LIST))
-	    display_privs(snl, list_pw ? list_pw : sudo_user.pw);
+    log_allowed(validated);
+    if (ISSET(sudo_mode, MODE_CHECK))
+	rval = display_cmnd(snl, list_pw ? list_pw : sudo_user.pw);
+    else if (ISSET(sudo_mode, MODE_LIST))
+	display_privs(snl, list_pw ? list_pw : sudo_user.pw);
 
-	/* Cleanup sudoers sources */
-	tq_foreach_fwd(snl, nss)
-	    nss->close(nss);
-
-#ifdef USING_NONUNIX_GROUPS
-	/* Finished with the groupcheck code */
-	sudo_nonunix_groupcheck_cleanup();
-#endif
-
-	/* Deferred exit due to sudo_ldap_close() */
-	if (ISSET(sudo_mode, (MODE_VALIDATE|MODE_CHECK|MODE_LIST))) {
-	    if (list_pw != NULL)
-		pw_delref(list_pw);
-	    exit(rc);
-	}
-
-	/* Must audit before uid change. */
-	audit_success(NewArgv);
-
-	if (ISSET(sudo_mode, MODE_LOGIN_SHELL)) {
-	    char *p;
-
-	    /* Convert /bin/sh -> -sh so shell knows it is a login shell */
-	    if ((p = strrchr(NewArgv[0], '/')) == NULL)
-		p = NewArgv[0];
-	    *p = '-';
-	    NewArgv[0] = p;
-
-	    /*
-	     * Newer versions of bash require the --login option to be used
-	     * in conjunction with the -c option even if the shell name starts
-	     * with a '-'.  Unfortunately, bash 1.x uses -login, not --login
-	     * so this will cause an error for that.
-	     */
-	    if (NewArgc > 1 && strcmp(NewArgv[0], "-bash") == 0) {
-		/* Use an extra slot before NewArgv so we can store --login. */
-		NewArgv--;
-		NewArgc++;
-		NewArgv[0] = NewArgv[1];
-		NewArgv[1] = "--login";
-	    }
-
-#if defined(__linux__) || defined(_AIX)
-	    /* Insert system-wide environment variables. */
-	    read_env_file(_PATH_ENVIRONMENT, TRUE);
-#endif
-	}
-
-	if (ISSET(sudo_mode, MODE_RUN)) {
-	    /* Insert system-wide environment variables. */
-	    if (def_env_file)
-		read_env_file(def_env_file, FALSE);
-
-	    /* Insert user-specified environment variables. */
-	    insert_env_vars(sudo_user.env_vars);
-	}
-
-	/* Restore signal handlers before we exec. */
-	(void) sigaction(SIGINT, &saved_sa_int, NULL);
-	(void) sigaction(SIGQUIT, &saved_sa_quit, NULL);
-	(void) sigaction(SIGTSTP, &saved_sa_tstp, NULL);
-
-	if (ISSET(sudo_mode, MODE_EDIT)) {
-	    exit(sudo_edit(NewArgc, NewArgv, envp));
-	} else {
-	    exit(run_command(safe_cmnd, NewArgv, env_get(), runas_pw->pw_uid,
-		CMND_WAIT));
-	}
-    } else if (ISSET(validated, FLAG_NO_USER | FLAG_NO_HOST)) {
-	audit_failure(NewArgv, "No user or host");
-	log_denial(validated, 1);
-	exit(1);
-    } else {
-	if (def_path_info) {
-	    /*
-	     * We'd like to not leak path info at all here, but that can
-	     * *really* confuse the users.  To really close the leak we'd
-	     * have to say "not allowed to run foo" even when the problem
-	     * is just "no foo in path" since the user can trivially set
-	     * their path to just contain a single dir.
-	     */
-	    log_denial(validated,
-		!(cmnd_status == NOT_FOUND_DOT || cmnd_status == NOT_FOUND));
-	    if (cmnd_status == NOT_FOUND)
-		warningx("%s: command not found", user_cmnd);
-	    else if (cmnd_status == NOT_FOUND_DOT)
-		warningx("ignoring `%s' found in '.'\nUse `sudo ./%s' if this is the `%s' you wish to run.", user_cmnd, user_cmnd, user_cmnd);
-	} else {
-	    /* Just tell the user they are not allowed to run foo. */
-	    log_denial(validated, 1);
-	}
-	audit_failure(NewArgv, "validation failure");
-	exit(1);
+    /* Cleanup sudoers sources */
+    tq_foreach_fwd(snl, nss) {
+	nss->close(nss);
     }
-    exit(0);	/* not reached */
+#ifdef USING_NONUNIX_GROUPS
+    /* Finished with the groupcheck code */
+    sudo_nonunix_groupcheck_cleanup();
+#endif
+
+    if (ISSET(sudo_mode, (MODE_VALIDATE|MODE_CHECK|MODE_LIST))) {
+	/* rval already set appropriately */
+	goto done;
+    }
+
+    /* Must audit before uid change. */
+    audit_success(NewArgv);
+
+    if (ISSET(sudo_mode, MODE_LOGIN_SHELL)) {
+	char *p;
+
+	/* Convert /bin/sh -> -sh so shell knows it is a login shell */
+	if ((p = strrchr(NewArgv[0], '/')) == NULL)
+	    p = NewArgv[0];
+	*p = '-';
+	NewArgv[0] = p;
+
+	/*
+	 * Newer versions of bash require the --login option to be used
+	 * in conjunction with the -c option even if the shell name starts
+	 * with a '-'.  Unfortunately, bash 1.x uses -login, not --login
+	 * so this will cause an error for that.
+	 */
+	if (NewArgc > 1 && strcmp(NewArgv[0], "-bash") == 0) {
+	    /* Use an extra slot before NewArgv so we can store --login. */
+	    NewArgv--;
+	    NewArgc++;
+	    NewArgv[0] = NewArgv[1];
+	    NewArgv[1] = "--login";
+	}
+
+#if defined(_AIX) || (defined(__linux__) && !defined(HAVE_PAM))
+	/* Insert system-wide environment variables. */
+	read_env_file(_PATH_ENVIRONMENT, TRUE);
+#endif
+#ifdef HAVE_LOGIN_CAP_H
+	/* Set environment based on login class. */
+	if (login_class) {
+	    login_cap_t *lc = login_getclass(login_class);
+	    if (lc != NULL) {
+		setusercontext(lc, runas_pw, runas_pw->pw_uid,
+		    LOGIN_SETPATH|LOGIN_SETENV);
+		login_close(lc);
+	    }
+	}
+#endif /* HAVE_LOGIN_CAP_H */
+    }
+
+    if (ISSET(sudo_mode, MODE_RUN)) {
+	/* Insert system-wide environment variables. */
+	if (def_env_file)
+	    read_env_file(def_env_file, FALSE);
+
+	/* Insert user-specified environment variables. */
+	insert_env_vars(sudo_user.env_vars);
+    }
+
+    /* Restore signal handlers before we exec. */
+    (void) sigaction(SIGINT, &saved_sa_int, NULL);
+    (void) sigaction(SIGQUIT, &saved_sa_quit, NULL);
+    (void) sigaction(SIGTSTP, &saved_sa_tstp, NULL);
+
+    if (ISSET(sudo_mode, MODE_EDIT)) {
+	exit(sudo_edit(NewArgc, NewArgv, envp));
+    } else {
+	exit(run_command(safe_cmnd, NewArgv, env_get(), runas_pw->pw_uid,
+	    CMND_WAIT));
+    }
+
+bad:
+    rval = FALSE;
+
+done:
+    cleanup(0);
+    exit(!rval);
 }
 
 /*
@@ -676,9 +685,9 @@ init_vars(envp)
 	if (sudo_mode == MODE_KILL || sudo_mode == MODE_INVALIDATE)
 	    errorx(1, "unknown uid: %u", (unsigned int) uid);
 
-	/* Need to make a fake struct passwd for the call to log_error(). */
+	/* Need to make a fake struct passwd for the call to log_fatal(). */
 	sudo_user.pw = sudo_fakepwuid(uid, gid);
-	log_error(0, "unknown uid: %u", (unsigned int) uid);
+	log_fatal(0, "unknown uid: %u", (unsigned int) uid);
     }
 #ifdef HAVE_MBR_CHECK_MEMBERSHIP
     mbr_uid_to_uuid(user_uid, user_uuid);
@@ -686,18 +695,18 @@ init_vars(envp)
     if (user_shell == NULL || *user_shell == '\0')
 	user_shell = estrdup(sudo_user.pw->pw_shell);
 
-    /* It is now safe to use log_error() and set_perms() */
+    /* It is now safe to use log_fatal() and set_perms() */
 
 #ifdef HAVE_GETGROUPS
     if ((user_ngroups = getgroups(0, NULL)) > 0) {
 	user_groups = emalloc2(user_ngroups, sizeof(GETGROUPS_T));
 	if (getgroups(user_ngroups, user_groups) < 0)
-	    log_error(USE_ERRNO|MSG_ONLY, "can't get group vector");
+	    log_fatal(USE_ERRNO|MSG_ONLY, "can't get group vector");
     }
 #endif
 
     if (nohostname)
-	log_error(USE_ERRNO|MSG_ONLY, "can't get hostname");
+	log_fatal(USE_ERRNO|MSG_ONLY, "can't get hostname");
 
     /*
      * Get current working directory.  Try as user, fall back to root.
@@ -717,16 +726,12 @@ init_vars(envp)
      * such as "list", we need to redo NewArgv and NewArgc.
      */
     if (ISSET(sudo_mode, MODE_SHELL)) {
-	char **av;
+	char **av, *cmnd = NULL;
+	int ac = 1;
 
-	/* Allocate 2 extra slots for --login and execve() failure (ENOEXEC). */
-	av = (char **) emalloc2(6, sizeof(char *));
-	av += 2;
-
-	av[0] = user_shell;	/* may be updated later */
 	if (NewArgc > 0) {
 	    /* shell -c "command" */
-	    char *cmnd, *src, *dst;
+	    char *src, *dst;
 	    size_t cmnd_size = (size_t) (NewArgv[NewArgc - 1] - NewArgv[0]) +
 		    strlen(NewArgv[NewArgc - 1]) + 1;
 
@@ -744,12 +749,19 @@ init_vars(envp)
 		dst--;	/* replace last space with a NUL */
 	    *dst = '\0';
 
+	    ac += 2; /* -c cmnd */
+	}
+
+	/* Allocate 2 extra slots for --login and execve() failure (ENOEXEC). */
+	av = (char **) emalloc2(ac + 3, sizeof(char *));
+	av[0] = user_shell;	/* may be updated later */
+	if (cmnd != NULL) {
 	    av[1] = "-c";
 	    av[2] = cmnd;
-	    NewArgc = 2;
 	}
-	av[++NewArgc] = NULL;
+	av[ac] = NULL;
 	NewArgv = av;
+	NewArgc = ac;
     } else if (ISSET(sudo_mode, MODE_EDIT) || NewArgc == 0) {
 	NewArgv--;
 	NewArgc++;
@@ -773,7 +785,7 @@ set_cmnd(sudo_mode)
 
     /* Resolve the path and return. */
     rval = FOUND;
-    user_stat = emalloc(sizeof(struct stat));
+    user_stat = ecalloc(1, sizeof(struct stat));
     if (sudo_mode & (MODE_RUN | MODE_EDIT | MODE_CHECK)) {
 	if (ISSET(sudo_mode, MODE_RUN | MODE_CHECK)) {
 	    if (def_secure_path && !user_is_exempt())
@@ -838,7 +850,7 @@ set_cmnd(sudo_mode)
 	user_base = user_cmnd;
 
     if (!update_defaults(SETDEF_CMND))
-	log_error(NO_STDERR|NO_EXIT, "problem with defaults entries");
+	log_error(NO_STDERR, "problem with defaults entries");
 
     return rval;
 }
@@ -998,76 +1010,90 @@ open_sudoers(sudoers, doedit, keepopen)
     int doedit;
     int *keepopen;
 {
-    struct stat statbuf;
+    struct stat sb;
     FILE *fp = NULL;
-    int rootstat;
 
-    /*
-     * Fix the mode and group on sudoers file from old default.
-     * Only works if file system is readable/writable by root.
-     */
-    if ((rootstat = stat_sudoers(sudoers, &statbuf)) == 0 &&
-	SUDOERS_UID == statbuf.st_uid && SUDOERS_MODE != 0400 &&
-	(statbuf.st_mode & 0007777) == 0400) {
-
-	if (chmod(sudoers, SUDOERS_MODE) == 0) {
-	    warningx("fixed mode on %s", sudoers);
-	    SET(statbuf.st_mode, SUDOERS_MODE);
-	    if (statbuf.st_gid != SUDOERS_GID) {
-		if (chown(sudoers, (uid_t) -1, SUDOERS_GID) == 0) {
-		    warningx("set group on %s", sudoers);
-		    statbuf.st_gid = SUDOERS_GID;
-		} else
-		    warning("unable to set group on %s", sudoers);
-	    }
-	} else
-	    warning("unable to fix mode on %s", sudoers);
-    }
-
-    /*
-     * Sanity checks on sudoers file.  Must be done as sudoers
-     * file owner.  We already did a stat as root, so use that
-     * data if we can't stat as sudoers file owner.
-     */
     set_perms(PERM_SUDOERS);
 
-    if (rootstat != 0 && stat_sudoers(sudoers, &statbuf) != 0)
-	log_error(USE_ERRNO|NO_EXIT, "can't stat %s", sudoers);
-    else if (!S_ISREG(statbuf.st_mode))
-	log_error(NO_EXIT, "%s is not a regular file", sudoers);
-    else if ((statbuf.st_mode & 07577) != (SUDOERS_MODE & 07577))
-	log_error(NO_EXIT, "%s is mode 0%o, should be 0%o", sudoers,
-	    (unsigned int) (statbuf.st_mode & 07777),
-	    (unsigned int) SUDOERS_MODE);
-    else if (statbuf.st_uid != SUDOERS_UID)
-	log_error(NO_EXIT, "%s is owned by uid %u, should be %u", sudoers,
-	    (unsigned int) statbuf.st_uid, (unsigned int) SUDOERS_UID);
-    else if (statbuf.st_gid != SUDOERS_GID && ISSET(statbuf.st_mode, S_IRGRP|S_IWGRP))
-	log_error(NO_EXIT, "%s is owned by gid %u, should be %u", sudoers,
-	    (unsigned int) statbuf.st_gid, (unsigned int) SUDOERS_GID);
-    else if ((fp = fopen(sudoers, "r")) == NULL)
-	log_error(USE_ERRNO|NO_EXIT, "can't open %s", sudoers);
-    else {
-	/*
-	 * Make sure we can actually read sudoers so we can present the
-	 * user with a reasonable error message (unlike the lexer).
-	 */
-	if (statbuf.st_size != 0 && fgetc(fp) == EOF) {
-	    log_error(USE_ERRNO|NO_EXIT, "can't read %s", sudoers);
-	    fclose(fp);
-	    fp = NULL;
-	}
-    }
-
-    if (fp != NULL) {
-	rewind(fp);
-	(void) fcntl(fileno(fp), F_SETFD, 1);
+    switch (sudo_secure_file(sudoers, SUDOERS_UID, SUDOERS_GID, &sb)) {
+	case SUDO_PATH_SECURE:
+	    /*
+	     * If we are expecting sudoers to be group readable but
+	     * it is not, we must open the file as root, not uid 1.
+	     */
+	    if (SUDOERS_UID == ROOT_UID && (SUDOERS_MODE & S_IRGRP)) {
+		if ((sb.st_mode & S_IRGRP) == 0)
+		    set_perms(PERM_ROOT);
+	    }
+	    /*
+	     * Open sudoers and make sure we can read it so we can present
+	     * the user with a reasonable error message (unlike the lexer).
+	     */
+	    if ((fp = fopen(sudoers, "r")) == NULL) {
+		log_error(USE_ERRNO, "unable to open %s", sudoers);
+	    } else {
+		if (sb.st_size != 0 && fgetc(fp) == EOF) {
+		    log_error(USE_ERRNO, "unable to read %s",
+			sudoers);
+		    fclose(fp);
+		    fp = NULL;
+		} else {
+		    /* Rewind fp and set close on exec flag. */
+		    rewind(fp);
+		    (void) fcntl(fileno(fp), F_SETFD, 1);
+		}
+	    }
+	    break;
+	case SUDO_PATH_MISSING:
+	    log_error(USE_ERRNO, "unable to stat %s", sudoers);
+	    break;
+	case SUDO_PATH_BAD_TYPE:
+	    log_error(0, "%s is not a regular file", sudoers);
+	    break;
+	case SUDO_PATH_WRONG_OWNER:
+	    log_error(0, "%s is owned by uid %u, should be %u",
+		sudoers, (unsigned int) sb.st_uid, (unsigned int) SUDOERS_UID);
+	    break;
+	case SUDO_PATH_WORLD_WRITABLE:
+	    log_error(0, "%s is world writable", sudoers);
+	    break;
+	case SUDO_PATH_GROUP_WRITABLE:
+	    log_error(0, "%s is owned by gid %u, should be %u",
+		sudoers, (unsigned int) sb.st_gid, (unsigned int) SUDOERS_GID);
+	    break;
+	default:
+	    /* NOTREACHED */
+	    break;
     }
 
     set_perms(PERM_ROOT);		/* change back to root */
+
     return fp;
 }
 
+static void
+sudo_check_suid(path)
+    const char *path;
+{
+    struct stat sb;
+
+    if (geteuid() != 0) {
+	if (strchr(path, '/') != NULL && stat(path, &sb) == 0) {
+	    /* Try to determine why sudo was not running as root. */
+	    if (sb.st_uid != ROOT_UID || !ISSET(sb.st_mode, S_ISUID)) {
+		errorx(1,
+		    "%s must be owned by uid %d and have the setuid bit set",
+		    path, ROOT_UID);
+	    } else {
+		errorx(1, "effective uid is not %d, is %s on a file system with the 'nosuid' option set or an NFS file system without root privileges?", ROOT_UID, path);
+	    }
+	} else {
+	    errorx(1, "effective uid is not %d, is sudo installed setuid root?",
+		ROOT_UID);
+	}
+    }
+}
+
 /*
  * Close all open files (except std*) and turn off core dumps.
  * Also sets the set_perms() pointer to the correct function.
@@ -1081,9 +1107,10 @@ initial_setup()
     struct rlimit rl;
 #endif
 
-    /* Reset signal mask. */
+    /* Reset signal mask and save signal state. */
     (void) sigemptyset(&mask);
     (void) sigprocmask(SIG_SETMASK, &mask, NULL);
+    save_signals();
 
 #if defined(__linux__)
     /*
@@ -1135,21 +1162,11 @@ static void
 set_loginclass(pw)
     struct passwd *pw;
 {
-    int errflags;
+    const int errflags = NO_MAIL|MSG_ONLY;
 
     if (!def_use_loginclass)
 	return;
 
-    /*
-     * Don't make it a fatal error if the user didn't specify the login
-     * class themselves.  We do this because if login.conf gets
-     * corrupted we want the admin to be able to use sudo to fix it.
-     */
-    if (login_class)
-	errflags = NO_MAIL|MSG_ONLY;
-    else
-	errflags = NO_MAIL|MSG_ONLY|NO_EXIT;
-
     if (login_class && strcmp(login_class, "-") != 0) {
 	if (user_uid != 0 &&
 	    strcmp(runas_user ? runas_user : def_runas_default, "root") != 0)
@@ -1164,7 +1181,15 @@ set_loginclass(pw)
     /* Make sure specified login class is valid. */
     lc = login_getclass(login_class);
     if (!lc || !lc->lc_class || strcmp(lc->lc_class, login_class) != 0) {
-	log_error(errflags, "unknown login class: %s", login_class);
+	/*
+	 * Don't make it a fatal error if the user didn't specify the login
+	 * class themselves.  We do this because if login.conf gets
+	 * corrupted we want the admin to be able to use sudo to fix it.
+	 */
+	if (login_class)
+	    log_fatal(errflags, "unknown login class: %s", login_class);
+	else
+	    log_error(errflags, "unknown login class: %s", login_class);
 	def_use_loginclass = FALSE;
     }
 }
@@ -1176,6 +1201,10 @@ set_loginclass(pw)
 }
 #endif /* HAVE_LOGIN_CAP_H */
 
+#ifndef AI_FQDN
+# define AI_FQDN AI_CANONNAME
+#endif
+
 /*
  * Look up the fully qualified domain name and set user_host and user_shost.
  */
@@ -1192,13 +1221,12 @@ set_fqdn()
 #ifdef HAVE_GETADDRINFO
     zero_bytes(&hint, sizeof(hint));
     hint.ai_family = PF_UNSPEC;
-    hint.ai_flags = AI_CANONNAME;
+    hint.ai_flags = AI_FQDN;
     if (getaddrinfo(user_host, NULL, &hint, &res0) != 0) {
 #else
     if (!(hp = gethostbyname(user_host))) {
 #endif
-	log_error(MSG_ONLY|NO_EXIT,
-	    "unable to resolve host %s", user_host);
+	log_error(MSG_ONLY, "unable to resolve host %s", user_host);
     } else {
 	if (user_shost != user_host)
 	    efree(user_shost);
@@ -1209,13 +1237,13 @@ set_fqdn()
 #else
 	user_host = estrdup(hp->h_name);
 #endif
-    }
-    if ((p = strchr(user_host, '.'))) {
-	*p = '\0';
-	user_shost = estrdup(user_host);
-	*p = '.';
-    } else {
-	user_shost = user_host;
+	if ((p = strchr(user_host, '.'))) {
+	    *p = '\0';
+	    user_shost = estrdup(user_host);
+	    *p = '.';
+	} else {
+	    user_shost = user_host;
+	}
     }
 }
 
@@ -1235,7 +1263,7 @@ set_runaspw(user)
     } else {
 	if ((runas_pw = sudo_getpwnam(user)) == NULL) {
 	    audit_failure(NewArgv, "unknown user: %s", user);
-	    log_error(NO_MAIL|MSG_ONLY, "unknown user: %s", user);
+	    log_fatal(NO_MAIL|MSG_ONLY, "unknown user: %s", user);
 	}
     }
 }
@@ -1255,7 +1283,7 @@ set_runasgr(group)
 	    runas_gr = sudo_fakegrnam(group);
     } else {
 	if ((runas_gr = sudo_getgrnam(group)) == NULL)
-	    log_error(NO_MAIL|MSG_ONLY, "unknown group: %s", group);
+	    log_fatal(NO_MAIL|MSG_ONLY, "unknown group: %s", group);
     }
 }
 
@@ -1295,7 +1323,9 @@ cleanup(gotsignal)
 	io_log_close();
 #endif
     }
-    term_restore(STDIN_FILENO, 0);
+#ifdef _PATH_SUDO_IO_LOGDIR
+    cleanup_pty(gotsignal);
+#endif
 #ifdef HAVE_SELINUX
     selinux_restore_tty();
 #endif
