@@ -80,6 +80,7 @@ struct command_status {
 #define CMD_ERRNO 1
 #define CMD_WSTATUS 2
 #define CMD_SIGNO 3
+#define CMD_PID 4
     int type;
     int val;
 };
@@ -96,6 +97,9 @@ struct command_status {
 #define FLAG_NO_USER		0x020
 #define FLAG_NO_HOST		0x040
 #define FLAG_NO_CHECK		0x080
+#define FLAG_NON_INTERACTIVE	0x100
+#define FLAG_BAD_PASSWORD	0x200
+#define FLAG_AUTH_ERROR		0x400
 
 /*
  * Pseudo-boolean values
@@ -183,6 +187,13 @@ struct command_status {
 #define user_role		(sudo_user.role)
 #define user_type		(sudo_user.type)
 
+#ifdef __TANDEM
+# define ROOT_UID	65535
+#else
+# define ROOT_UID	0
+#endif
+#define ROOT_GID	0
+
 /*
  * We used to use the system definition of PASS_MAX or _PASSWD_LEN,
  * but that caused problems with various alternate authentication
@@ -219,13 +230,14 @@ void aix_restoreauthdb __P((void));
 int get_boottime __P((struct timeval *));
 
 /* check.c */
+int check_user		__P((int, int));
 int user_is_exempt	__P((void));
-void check_user		__P((int, int));
 void remove_timestamp	__P((int));
 
 /* env.c */
 char **env_get		__P((void));
 void env_init		__P((int lazy));
+void env_merge		__P((char * const envp[], int overwrite));
 void init_envtables	__P((void));
 void insert_env_vars	__P((struct list_member *));
 void read_env_file	__P((const char *, int));
@@ -238,6 +250,9 @@ int sudo_execve __P((const char *path, char *argv[], char *envp[], uid_t uid,
 void save_signals __P((void));
 void restore_signals __P((void));
 
+/* exec_pty.c */
+void cleanup_pty __P((int gotsignal));
+
 /* fileops.c */
 char *sudo_parseln	__P((FILE *));
 int lock_file		__P((int, int));
@@ -325,11 +340,11 @@ RETSIGTYPE cleanup	__P((int));
 void set_fqdn		__P((void));
 
 /* sudo_auth.c */
-void verify_user	__P((struct passwd *, char *));
-void pass_warn		__P((FILE *));
+int sudo_auth_cleanup	__P((struct passwd *));
+int sudo_auth_init	__P((struct passwd *));
+int verify_user		__P((struct passwd *, char *, int));
 void dump_auth_methods	__P((void));
-void sudo_auth_init	__P((struct passwd *));
-void sudo_auth_cleanup	__P((struct passwd *));
+void pass_warn		__P((FILE *));
 
 /* sudo_nss.c */
 void display_privs	__P((struct sudo_nss_list *, struct passwd *));
