@@ -87,6 +87,10 @@ sudo_read_nss()
 		/* NOTFOUND affects the most recent entry */
 		tq_last(&snl)->ret_if_notfound = TRUE;
 		got_match = FALSE;
+	    } else if (strcasecmp(cp, "[SUCCESS=return]") == 0 && got_match) {
+		/* SUCCESS affects the most recent entry */
+		tq_last(&snl)->ret_if_found = TRUE;
+		got_match = FALSE;
 	    } else
 		got_match = FALSE;
 	}
@@ -215,13 +219,13 @@ reset_groups(pw)
 	aix_setauthdb(pw->pw_name);
 # endif
 	if (initgroups(pw->pw_name, pw->pw_gid) == -1)
-	    log_error(USE_ERRNO|MSG_ONLY, "can't reset group vector");
+	    log_fatal(USE_ERRNO|MSG_ONLY, "can't reset group vector");
 	efree(user_groups);
 	user_groups = NULL;
 	if ((user_ngroups = getgroups(0, NULL)) > 0) {
 	    user_groups = emalloc2(user_ngroups, sizeof(GETGROUPS_T));
 	    if (getgroups(user_ngroups, user_groups) < 0)
-		log_error(USE_ERRNO|MSG_ONLY, "can't get group vector");
+		log_fatal(USE_ERRNO|MSG_ONLY, "can't get group vector");
 	}
 # ifdef HAVE_SETAUTHDB
 	aix_restoreauthdb();
@@ -288,13 +292,14 @@ display_privs(snl, pw)
     tq_foreach_fwd(snl, nss) {
 	count += nss->display_privs(nss, pw, &privs);
     }
-    if (count) {
-	lbuf_print(&defs);
-	lbuf_print(&privs);
-    } else {
-	printf("User %s is not allowed to run sudo on %s.\n", pw->pw_name,
-	    user_shost);
+    if (count == 0) {
+	defs.len = 0;
+	privs.len = 0;
+	lbuf_append(&privs, "User %s is not allowed to run sudo on %s.\n",
+	    pw->pw_name, user_shost);
     }
+    lbuf_print(&defs);
+    lbuf_print(&privs);
 
     lbuf_destroy(&defs);
     lbuf_destroy(&privs);
@@ -316,7 +321,7 @@ display_cmnd(snl, pw)
 
     tq_foreach_fwd(snl, nss) {
 	if (nss->display_cmnd(nss, pw) == 0)
-	    return 0;
+	    return TRUE;
     }
-    return 1;
+    return FALSE;
 }
