@@ -228,6 +228,9 @@ sudo_askpass(prompt)
     (void) close(pfd[0]);
     (void) sigaction(SIGPIPE, &saved_sa_pipe, NULL);
 
+    if (pass == NULL)
+	errno = EINTR;	/* make cancel button simulate ^C */
+
     return pass;
 }
 
@@ -272,8 +275,7 @@ getln(fd, buf, bufsiz, feedback)
 		}
 		continue;
 	    }
-	    if (write(fd, "*", 1) == -1)
-		/* shut up glibc */;
+	    ignore_result(write(fd, "*", 1));
 	}
 	*cp++ = c;
     }
