@@ -185,9 +185,8 @@ main(argc, argv)
 		usage(1);
 	}
     }
-    argc -= optind;
-    argv += optind;
-    if (argc)
+    /* There should be no other command line arguments. */
+    if (argc - optind != 0)
 	usage(1);
 
     sudo_setpwent();
@@ -452,6 +451,9 @@ reparse_sudoers(editor, args, strict, quiet)
     FILE *fp;
     int ch;
 
+    if (tq_empty(&sudoerslist))
+	return TRUE;
+
     /*
      * Parse the edited sudoers files and do sanity checking
      */
@@ -477,10 +479,10 @@ reparse_sudoers(editor, args, strict, quiet)
 	}
 	fclose(yyin);
 	if (!parse_error) {
-	    if (!update_defaults(SETDEF_GENERIC|SETDEF_HOST|SETDEF_USER) ||
+	    if (!check_defaults(SETDEF_ALL, quiet) ||
 		check_aliases(strict, quiet) != 0) {
 		parse_error = TRUE;
-		errorfile = sp->path;
+		errorfile = NULL;
 	    }
 	}
 
@@ -501,10 +503,11 @@ reparse_sudoers(editor, args, strict, quiet)
 	    tq_foreach_fwd(&sudoerslist, sp) {
 		if (errorfile == NULL || strcmp(sp->path, errorfile) == 0) {
 		    edit_sudoers(sp, editor, args, errorlineno);
-		    break;
+		    if (errorfile != NULL)
+			break;
 		}
 	    }
-	    if (sp == NULL)
+	    if (errorfile != NULL && sp == NULL)
 		errorx(1, "internal error, can't find %s in list!", sudoers);
 	}
 
@@ -538,9 +541,9 @@ install_sudoers(sp, oldperms)
 	(void) unlink(sp->tpath);
 	if (!oldperms && fstat(sp->fd, &sb) != -1) {
 	    if (sb.st_uid != SUDOERS_UID || sb.st_gid != SUDOERS_GID)
-		(void) chown(sp->path, SUDOERS_UID, SUDOERS_GID);
+		ignore_result(chown(sp->path, SUDOERS_UID, SUDOERS_GID));
 	    if ((sb.st_mode & 0777) != SUDOERS_MODE)
-		(void) chmod(sp->path, SUDOERS_MODE);
+		ignore_result(chmod(sp->path, SUDOERS_MODE));
 	}
 	return TRUE;
     }
@@ -775,9 +778,12 @@ check_syntax(sudoers_path, quiet, strict)
 	parse_error = TRUE;
 	errorfile = sudoers_path;
     }
-    if (!parse_error && check_aliases(strict, quiet) != 0) {
-	parse_error = TRUE;
-	errorfile = sudoers_path;
+    if (!parse_error) {
+	if (!check_defaults(SETDEF_ALL, quiet) ||
+	    check_aliases(strict, quiet) != 0) {
+	    parse_error = TRUE;
+	    errorfile = NULL;
+	}
     }
     error = parse_error;
     if (!quiet) {
@@ -785,7 +791,7 @@ check_syntax(sudoers_path, quiet, strict)
 	    if (errorlineno != -1)
 		(void) printf("parse error in %s near line %d\n", errorfile,
 		    errorlineno);
-	    else
+	    else if (errorfile != NULL)
 		(void) printf("parse error in %s\n", errorfile);
 	} else {
 	    struct sudoersfile *sp;
@@ -846,13 +852,13 @@ open_sudoers(path, doedit, keepopen)
 	    break;
     }
     if (entry == NULL) {
-	entry = emalloc(sizeof(*entry));
+	entry = ecalloc(1, sizeof(*entry));
 	entry->path = estrdup(path);
-	entry->modified = 0;
+	/* entry->modified = 0; */
 	entry->prev = entry;
-	entry->next = NULL;
+	/* entry->next = NULL; */
 	entry->fd = open(entry->path, open_flags, SUDOERS_MODE);
-	entry->tpath = NULL;
+	/* entry->tpath = NULL; */
 	entry->doedit = doedit;
 	if (entry->fd == -1) {
 	    warning("%s", entry->path);
@@ -1249,9 +1255,8 @@ quit(signo)
 {
     cleanup(signo);
 #define	emsg	 " exiting due to signal.\n"
-    if (write(STDERR_FILENO, getprogname(), strlen(getprogname())) == -1 ||
-	write(STDERR_FILENO, emsg, sizeof(emsg) - 1) == -1)
-	/* shut up glibc */;
+    ignore_result(write(STDERR_FILENO, getprogname(), strlen(getprogname())));
+    ignore_result(write(STDERR_FILENO, emsg, sizeof(emsg) - 1));
     _exit(signo);
 }
 
